import numpy as np

from AvenirCommon.Util import GBRange
import SpectrumCommon.Const.TB as tbc
import SpectrumCommon.External.TB.TBFortWrapper as fort



def create_notification_defaults_2022(ISO3, db_notification, Noti_Distribution, first_idx):

    notif_base = db_notification[first_idx]
    
    notif_base_split =  np.zeros((tbc.TB_NotificationCaseCount))
    notif_base_pct = np.zeros((tbc.TB_NotificationCaseCount))
    
    new_cases = Noti_Distribution['new_labconf_num']+Noti_Distribution['new_clindx_num']+Noti_Distribution['new_ep_num'] 
    relapse_cases = Noti_Distribution['ret_rel_labconf_num']+Noti_Distribution['ret_rel_clindx_num']+Noti_Distribution['ret_rel_ep_num']
    ret_nrel_cases = Noti_Distribution['ret_nrel_num'] 
    
    c_ep = (Noti_Distribution['new_ep_num']+Noti_Distribution['ret_rel_ep_num'])/(new_cases+relapse_cases) if (new_cases+relapse_cases)>0 else 0
    bac_pos_pct = (Noti_Distribution['new_labconf_num']+Noti_Distribution['ret_rel_labconf_num'])/(new_cases + relapse_cases)if (new_cases+relapse_cases)>0 else 0
    
    total_cases =  new_cases + relapse_cases + ret_nrel_cases

    notif_scale = total_cases/notif_base
    notif_base = notif_base*notif_scale
    
    # fort_outputs =  get_tb_fort_outputs(ISO3, TB_FinalHist_Year, notif[0], fort_vsn)
    # notif[tbc.TB_slice] = fort_outputs['nMid'][fort_slice]

    notif_HIV           = (total_cases)*Noti_Distribution['newrel_hivpos_perc']
    notif_HIVpNeonates  = notif_HIV*Noti_Distribution['newrel_014_perc']/15
    notif_HIVp0_14      = notif_HIV*Noti_Distribution['newrel_014_perc']-notif_HIVpNeonates
    notif_HIVp15plus    = notif_HIV*Noti_Distribution['newrel_15plus_perc']


    notif_NotOnART      = notif_HIV*(1-Noti_Distribution['newrel_art_perc'])
    notif_OnART         = notif_HIV-notif_NotOnART
    
    notif_OnARTNeoNates = notif_OnART*Noti_Distribution['newrel_014_perc']/15
    notif_OnART0_14     = notif_OnART*Noti_Distribution['newrel_014_perc']-notif_OnARTNeoNates
    notif_OnART15plus   = notif_OnART*Noti_Distribution['newrel_15plus_perc']
    
    notif_base_split[tbc.TB_NewCases                    ] = new_cases
    notif_base_split[tbc.TB_NewBacterConfirmCases       ] = Noti_Distribution['new_labconf_num']
    notif_base_split[tbc.TB_NewClinicDiagCases          ] = Noti_Distribution['new_clindx_num']
    notif_base_split[tbc.TB_NewExtrapulmonaryCases      ] = Noti_Distribution['new_ep_num']
    notif_base_split[tbc.TB_RelapseCases                ] = relapse_cases
    notif_base_split[tbc.TB_RelapseBacterConfirmCases   ] = Noti_Distribution['ret_rel_labconf_num']
    notif_base_split[tbc.TB_RelapseClinicDiagCases      ] = Noti_Distribution['ret_rel_clindx_num']
    notif_base_split[tbc.TB_RelapseExtrapulmonaryCases  ] = Noti_Distribution['ret_rel_ep_num']
    notif_base_split[tbc.TB_NewAndRelapsedCases  ]        = new_cases+ relapse_cases
    
    notif_base_split[tbc.TB_PrevTreatExcludeRelapseCases] = ret_nrel_cases
    notif_base_split[tbc.TB_PrevTreatIncludeRelapseCases] = relapse_cases+ret_nrel_cases
    notif_base_split[tbc.TB_NewRelapse0_9Cases  ] = Noti_Distribution['newrel_04_num']+Noti_Distribution['newrel_59_num']
    notif_base_split[tbc.TB_NewRelapse10_14Cases] = Noti_Distribution['newrel_1014_num']
    notif_base_split[tbc.TB_NewRelapse15pCases  ] = Noti_Distribution['newrel_15plus_num']
    notif_base_split[tbc.TB_HIVpCases        ] = notif_HIV 
    notif_base_split[tbc.TB_HIVp15pCases     ] = notif_HIVp15plus
    notif_base_split[tbc.TB_HIVp0_14Cases    ] = notif_HIVp0_14
    notif_base_split[tbc.TB_HIVpNeonateCases ] = notif_HIVpNeonates
    notif_base_split[tbc.TB_NotOnARTCases    ] = notif_NotOnART
    notif_base_split[tbc.TB_OnARTCases       ] = notif_OnART
    notif_base_split[tbc.TB_OnART15pCases    ] = notif_OnARTNeoNates
    notif_base_split[tbc.TB_OnARRT0_14Cases  ] = notif_OnART0_14
    notif_base_split[tbc.TB_OnARTNeonateCases] = notif_OnARTNeoNates
    notif_base_split[tbc.TB_HIVpExcludeART] = 0

    notif_base_pct[tbc.TB_NewCases                    ] = notif_base_split[tbc.TB_NewCases                    ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewBacterConfirmCases       ] = notif_base_split[tbc.TB_NewBacterConfirmCases       ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewClinicDiagCases          ] = notif_base_split[tbc.TB_NewClinicDiagCases          ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewExtrapulmonaryCases      ] = notif_base_split[tbc.TB_NewExtrapulmonaryCases      ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_RelapseCases                ] = notif_base_split[tbc.TB_RelapseCases                ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_RelapseBacterConfirmCases   ] = notif_base_split[tbc.TB_RelapseBacterConfirmCases   ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_RelapseClinicDiagCases      ] = notif_base_split[tbc.TB_RelapseClinicDiagCases      ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_RelapseExtrapulmonaryCases  ] = notif_base_split[tbc.TB_RelapseExtrapulmonaryCases  ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewAndRelapsedCases  ]        = notif_base_split[tbc.TB_NewAndRelapsedCases         ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_PrevTreatExcludeRelapseCases] = notif_base_split[tbc.TB_PrevTreatExcludeRelapseCases]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_PrevTreatIncludeRelapseCases] = notif_base_split[tbc.TB_PrevTreatIncludeRelapseCases]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewRelapse0_9Cases  ] = notif_base_split[tbc.TB_NewRelapse0_9Cases  ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewRelapse10_14Cases] = notif_base_split[tbc.TB_NewRelapse10_14Cases]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NewRelapse15pCases  ] = notif_base_split[tbc.TB_NewRelapse15pCases  ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_HIVpCases        ] = notif_base_split[tbc.TB_HIVpCases        ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_HIVp15pCases     ] = notif_base_split[tbc.TB_HIVp15pCases     ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_HIVp0_14Cases    ] = notif_base_split[tbc.TB_HIVp0_14Cases    ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_HIVpNeonateCases ] = notif_base_split[tbc.TB_HIVpNeonateCases ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_NotOnARTCases    ] = notif_base_split[tbc.TB_NotOnARTCases    ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_OnARTCases       ] = notif_base_split[tbc.TB_OnARTCases       ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_OnART15pCases    ] = notif_base_split[tbc.TB_OnART15pCases    ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_OnARRT0_14Cases  ] = notif_base_split[tbc.TB_OnARRT0_14Cases  ]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_OnARTNeonateCases] = notif_base_split[tbc.TB_OnARTNeonateCases]/total_cases if total_cases >0 else 0
    notif_base_pct[tbc.TB_HIVpExcludeART   ] = notif_base_split[tbc.TB_NotOnARTCases    ]/total_cases if total_cases >0 else 0

    # notif_split = apply_notif_split(notif_split, notif, notif_pct)

    return notif_base_pct, notif_scale, c_ep, bac_pos_pct




def create_notification_defaults(ISO3, db_notification, Noti_Distribution, first_year, first_idx, final_idx, year_len, notif_base_pct, tb_slice, fort_slice, fort_vsn, notif_scale, subnat_ratio):
    notif = np.array(db_notification[first_idx:final_idx+1])*notif_scale
    
    notif_split =  np.full((tbc.TB_NotificationCaseCount, year_len), 0)
    notif_pct = np.zeros((tbc.TB_NotificationCaseCount))
    
    fort_outputs =  fort.get_tb_fort_outputs(ISO3, fort_vsn)
    notif[tb_slice] = [v*subnat_ratio for v in fort_outputs['nMid'][fort_slice]]
    last_default_year = tbc.TB_Base_Year if first_year<=tbc.TB_Base_Year else first_year
    for yr in range(first_year, last_default_year+1):
        t = yr-first_year
        total_cases = notif[t]
        
        notif_split[tbc.TB_NewCases                    ][t] = notif_base_pct[tbc.TB_NewCases]*total_cases
        notif_split[tbc.TB_NewBacterConfirmCases       ][t] = notif_base_pct[tbc.TB_NewBacterConfirmCases]*total_cases
        notif_split[tbc.TB_NewClinicDiagCases          ][t] = notif_base_pct[tbc.TB_NewClinicDiagCases]*total_cases
        notif_split[tbc.TB_NewExtrapulmonaryCases      ][t] = notif_base_pct[tbc.TB_NewExtrapulmonaryCases]*total_cases
        notif_split[tbc.TB_RelapseCases                ][t] = notif_base_pct[tbc.TB_RelapseCases]*total_cases
        notif_split[tbc.TB_RelapseBacterConfirmCases   ][t] = notif_base_pct[tbc.TB_RelapseBacterConfirmCases]*total_cases
        notif_split[tbc.TB_RelapseClinicDiagCases      ][t] = notif_base_pct[tbc.TB_RelapseClinicDiagCases]*total_cases
        notif_split[tbc.TB_RelapseExtrapulmonaryCases  ][t] = notif_base_pct[tbc.TB_RelapseExtrapulmonaryCases]*total_cases
        notif_split[tbc.TB_NewAndRelapsedCases  ][t]        = notif_base_pct[tbc.TB_NewAndRelapsedCases]*total_cases
        
        notif_split[tbc.TB_PrevTreatExcludeRelapseCases][t] = notif_base_pct[tbc.TB_PrevTreatExcludeRelapseCases]*total_cases
        notif_split[tbc.TB_PrevTreatIncludeRelapseCases][t] = notif_base_pct[tbc.TB_PrevTreatIncludeRelapseCases]*total_cases
        notif_split[tbc.TB_NewRelapse0_9Cases  ][t] = notif_base_pct[tbc.TB_NewRelapse0_9Cases  ]*total_cases
        notif_split[tbc.TB_NewRelapse10_14Cases][t] = notif_base_pct[tbc.TB_NewRelapse10_14Cases]*total_cases
        notif_split[tbc.TB_NewRelapse15pCases  ][t] = notif_base_pct[tbc.TB_NewRelapse15pCases  ]*total_cases
        notif_split[tbc.TB_HIVpCases        ][t] = notif_base_pct[tbc.TB_HIVpCases        ]*total_cases
        notif_split[tbc.TB_HIVp15pCases     ][t] = notif_base_pct[tbc.TB_HIVp15pCases     ]*total_cases
        notif_split[tbc.TB_HIVp0_14Cases    ][t] = notif_base_pct[tbc.TB_HIVp0_14Cases    ]*total_cases
        notif_split[tbc.TB_HIVpNeonateCases ][t] = notif_base_pct[tbc.TB_HIVpNeonateCases ]*total_cases
        notif_split[tbc.TB_NotOnARTCases    ][t] = notif_base_pct[tbc.TB_NotOnARTCases    ]*total_cases
        notif_split[tbc.TB_OnARTCases       ][t] = notif_base_pct[tbc.TB_OnARTCases       ]*total_cases
        notif_split[tbc.TB_OnART15pCases    ][t] = notif_base_pct[tbc.TB_OnART15pCases    ]*total_cases
        notif_split[tbc.TB_OnARRT0_14Cases  ][t] = notif_base_pct[tbc.TB_OnARRT0_14Cases  ]*total_cases
        notif_split[tbc.TB_OnARTNeonateCases][t] = notif_base_pct[tbc.TB_OnARTNeonateCases]*total_cases
        notif_split[tbc.TB_HIVpExcludeART   ][t] = notif_base_pct[tbc.TB_HIVpExcludeART]*total_cases

        new_cases = notif_split[tbc.TB_NewCases][t] 
        relapse_cases = notif_split[tbc.TB_RelapseCases][t]
        notif_HIV = notif_split[tbc.TB_HIVpCases][t]

        notif_pct[tbc.TB_NewCases                    ] = notif_split[tbc.TB_NewCases                    ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_NewBacterConfirmCases       ] = notif_split[tbc.TB_NewBacterConfirmCases       ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_NewClinicDiagCases          ] = notif_split[tbc.TB_NewClinicDiagCases          ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_NewExtrapulmonaryCases      ] = notif_split[tbc.TB_NewExtrapulmonaryCases      ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_RelapseCases                ] = notif_split[tbc.TB_RelapseCases                ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_RelapseBacterConfirmCases   ] = notif_split[tbc.TB_RelapseBacterConfirmCases   ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_RelapseClinicDiagCases      ] = notif_split[tbc.TB_RelapseClinicDiagCases      ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_RelapseExtrapulmonaryCases  ] = notif_split[tbc.TB_RelapseExtrapulmonaryCases  ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_NewAndRelapsedCases  ]        = notif_split[tbc.TB_NewAndRelapsedCases         ][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_PrevTreatExcludeRelapseCases] = notif_split[tbc.TB_PrevTreatExcludeRelapseCases][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_PrevTreatIncludeRelapseCases] = notif_split[tbc.TB_PrevTreatIncludeRelapseCases][t]/total_cases if total_cases > 0 else 0
        notif_pct[tbc.TB_NewRelapse0_9Cases  ] = notif_split[tbc.TB_NewRelapse0_9Cases  ][t]/(new_cases+relapse_cases) if (new_cases+relapse_cases) > 0 else 0
        notif_pct[tbc.TB_NewRelapse10_14Cases] = notif_split[tbc.TB_NewRelapse10_14Cases][t]/(new_cases+relapse_cases) if (new_cases+relapse_cases) > 0 else 0
        notif_pct[tbc.TB_NewRelapse15pCases  ] = notif_split[tbc.TB_NewRelapse15pCases  ][t]/(new_cases+relapse_cases) if (new_cases+relapse_cases) > 0 else 0
        notif_pct[tbc.TB_HIVpCases        ] = notif_split[tbc.TB_HIVpCases        ][t]/(new_cases+relapse_cases) if (new_cases+relapse_cases) > 0 else 0
        notif_pct[tbc.TB_HIVp15pCases     ] = notif_split[tbc.TB_HIVp15pCases     ][t]/notif_split[tbc.TB_HIVpCases][t] if notif_split[tbc.TB_HIVpCases][t] > 0 else 0
        notif_pct[tbc.TB_HIVp0_14Cases    ] = notif_split[tbc.TB_HIVp0_14Cases    ][t]/notif_split[tbc.TB_HIVpCases][t] if notif_split[tbc.TB_HIVpCases][t] > 0 else 0
        notif_pct[tbc.TB_HIVpNeonateCases ] = notif_split[tbc.TB_HIVpNeonateCases ][t]/notif_split[tbc.TB_HIVpCases][t] if notif_split[tbc.TB_HIVpCases][t] > 0 else 0
        notif_pct[tbc.TB_NotOnARTCases    ] = notif_split[tbc.TB_NotOnARTCases    ][t]/notif_split[tbc.TB_HIVpCases][t] if notif_split[tbc.TB_HIVpCases][t] > 0 else 0
        notif_pct[tbc.TB_OnARTCases       ] = notif_split[tbc.TB_OnARTCases       ][t]/notif_split[tbc.TB_HIVpCases][t] if notif_split[tbc.TB_HIVpCases][t] > 0 else 0
        notif_pct[tbc.TB_OnART15pCases    ] = 0.01 # notif_split[tbc.TB_OnART15pCases    ][0]/notif_split[tbc.TB_OnARTCases][0]
        notif_pct[tbc.TB_OnARRT0_14Cases  ] = 0.01 # notif_split[tbc.TB_OnARRT0_14Cases  ][0]/notif_split[tbc.TB_OnARTCases][0]
        notif_pct[tbc.TB_OnARTNeonateCases] = 0.01 # notif_split[tbc.TB_OnARTNeonateCases][0]/notif_split[tbc.TB_OnARTCases][0]
        notif_pct[tbc.TB_HIVpExcludeART   ] = notif_split[tbc.TB_NotOnARTCases    ][0]/(notif_split[tbc.TB_NotOnARTCases    ][0]+(total_cases-notif_HIV)) if (notif_split[tbc.TB_NotOnARTCases    ][0]+(total_cases-notif_HIV)) > 0 else 0

    notif_split = apply_notif_split(first_year, notif_split, notif, notif_pct)

    return notif, notif_split, notif_pct, fort_outputs

def apply_notif_split(first_year, notif_split, notif, notif_pct):
    base_t = tbc.TB_Base_Year-first_year+1
    for i in GBRange(0, tbc.TB_NotOnARTCases):
        for t in GBRange(base_t, len(notif_split[i])-1):
            notif_split[i][t] = notif[t]*notif_pct[i]

    for i in GBRange(tbc.TB_HIVpCases+1, tbc.TB_HIVpExcludeART):
        for t in GBRange(base_t, len(notif_split[i])-1):
            notif_split[i][t] = notif[t]*notif_pct[i]*notif_pct[tbc.TB_HIVpCases]

    return notif_split



# Fields used in IH data structures

IH_F_PROG_AREA_ID             = 'progAreaID'
IH_F_SUBGROUP_ID              = 'subgroupID'
IH_F_DELIV_CHAN_ID            = 'delivChanID'
IH_F_DELIV_CHAN_PACK_ID       = 'delivChanPackID'
IH_F_HEALTH_SYS_ADMIN_UNIT_ID = 'healthSysAdminUnitID'
IH_F_NAT_PROG_ID              = 'natProgID'

# programme area records

IH_PROG_AREA_ID                 = IH_F_PROG_AREA_ID  # string;  numeric mstID for defaults. GUID for customs.
IH_PROG_AREA_STR_CONSTANT       = 'stringConstant'   # string;  Defaults only.
IH_PROG_AREA_NAME               = 'name'             # string;  Holds custom name or overridden default name.
IH_PROG_AREA_ACTIVE             = 'active'           # boolean; Is user using this item or not?
IH_PROG_AREA_SUBGROUP_RECORDS   = 'subgroupRecords'  # list; Subgroup dicts
IH_PROG_AREA_CUSTOM             = 'custom'           # boolean; user-entered = true; default = false

# subgroup records (these sit inside the programme area records)

IH_SUBGROUP_ID           = IH_F_SUBGROUP_ID # string;  numeric mstID for defaults. GUID for customs.
IH_SUBGROUP_STR_CONSTANT = 'stringConstant' # string;  Defaults only.
IH_SUBGROUP_NAME         = 'name'           # string;  Holds custom name or overridden default name.
IH_SUBGROUP_ACTIVE       = 'active'         # boolean; Is user using this item or not?
IH_SUBGROUP_CUSTOM       = 'custom'         # boolean; user-entered = true; default = false

# delivery channel records

IH_DELIV_CHAN_ID           = IH_F_DELIV_CHAN_ID  # string;  numeric mstID for defaults. GUID for customs.
IH_DELIV_CHAN_STR_CONSTANT = 'stringConstant'    # string;  Defaults only.
IH_DELIV_CHAN_NAME         = 'name'              # string;  Holds custom name or overridden default name.
IH_DELIV_CHAN_ACTIVE       = 'active'            # boolean; Is user using this item or not?
IH_DELIV_CHAN_CUSTOM       = 'custom'            # boolean; user-entered = true; default = false

# delivery channel package records (Delivery Channel Mode only)

IH_DELIV_CHAN_PACK_ID     = IH_F_DELIV_CHAN_PACK_ID # string;  numeric mstID for defaults. GUID for customs.
IH_DELIV_CHAN_PACK_NAME   = 'name'                  # string;  Holds custom name or overridden default name.
IH_DELIV_CHAN_PACK_ACTIVE = 'active'                # boolean; For defaults only.
IH_DELIV_CHAN_PACK_CUSTOM = 'custom'                # boolean; user-entered = true; default = false

# national programme records (Delivery Channel Mode only)

IH_NAT_PROG_ID     = IH_F_NAT_PROG_ID # string;  numeric mstID for defaults. GUID for customs.
IH_NAT_PROG_NAME   = 'name'           # string;  Holds custom name or overridden default name.
IH_NAT_PROG_ACTIVE = 'active'         # boolean; For defaults only.
IH_NAT_PROG_CUSTOM = 'custom'         # boolean; user-entered = true; default = false

# aduit records (tool that is used when an IHT projection is loaded)

IH_AUDIT_STATUS_OVERRIDDEN = 'statusOverridden'
IH_AUDIT_COMMENT           = 'comment'

# health system administrative units

IH_HEALTH_SYS_ADMIN_UNIT_ID                 = IH_F_HEALTH_SYS_ADMIN_UNIT_ID  # string;  numeric mstID for defaults. GUID for customs.
IH_HEALTH_SYS_ADMIN_UNIT_STR_CONSTANT       = 'stringConstant'        # string;  Defaults only.
IH_HEALTH_SYS_ADMIN_UNIT_NAME               = 'name'                  # string;  Holds custom name or overridden default name.
IH_HEALTH_SYS_ADMIN_UNIT_ACTIVE             = 'active'                # boolean; Is user using this item or not?
IH_HEALTH_SYS_ADMIN_UNIT_CUSTOM             = 'custom'                # boolean; user-entered = true; default = false
IH_HEALTH_SYS_ADMIN_UNIT_NUMBER             = 'number'                # integer




# NOTE: If you use any of the constants below, please change the UH to IH before using them.
# This will signal that the constant is actually being used. Otherwise, the constant is just
# a carryover from the desktop.

# This unit declares constants used throughout the module and the other IHT
# modules. Try to use as many standard global constants from GB as possible
# (ex: GB_NOERROR, GB_MAX_PROJ).

from SpectrumCommon.Const.GB import *

# Database latest versions
IH_EXCHANGE_RATE_DB_CURR_VERSION = 'V2'

# Database names (also the names of the sheets in IHModData.xlsx)
IH_EXCHANGE_RATE_DB_NAME = 'ExchangeRateDB'

# Database directories for data by country. Files in these folders are named using the 3 letter iso country codes instead of the database names above.
IH_EXCHANGE_RATE_DB_DIR = r'ExchangeRate\countries'

# Data Import tool version
IH_DATA_IMPORT_TOOL_VERSION = 'V1'

# Any user-entered item must have an ID of 10000 or higher so the ID does not potentially overlap with defaults.
IH_MIN_CUSTOM_ITEM_ID = 10000

# Constants for writing to file 
UH_Version1       = 1.0
UH_Version2       = 1.1
UH_Version3       = 1.2
UH_Version4       = 1.3
UH_Version5       = 1.4
UH_DataFileTitle  = 'Unified Health Model (UHM) Data File'
UH_DataFileTitle2 = 'OneHealth Tool (OHT) Core Data File'
UH_MinColCount    = 12

# Locked projection constants 
UH_LockAreas = 1
UH_LoadAreas = 2

# Maximum number of regions the user can enter 
UH_MaxRegions = 100

# Maximum total number of interventions that the user can have in IHT. 
#UH_MaxTotalInterventions = UH_NumDefaultIntervs + UH_MaxCustomInterventions

# UHM Mode constants. 
UH_ProgAreaMode  = 1
UH_DelivChanMode = 2

# Default delivery channel IDs. 

IH_COMMUNITY_CHAN_CURR_ID             = 1
IH_OUTREACH_CHAN_CURR_ID              = 2
IH_PREHOSPITAL_EMERGENCY_CHAN_CURR_ID = 3
IH_GEN_OUTPATIENT_SERV_CHAN_CURR_ID   = 4
IH_FIRST_REFERRAL_CHAN_CURR_ID        = 5
IH_SEC_REFERRAL_CHAN_CURR_ID          = 6
IH_NUM_DEFAULT_DELIV_CHANS_COSTED     = IH_SEC_REFERRAL_CHAN_CURR_ID # Applies to IHT and TB but no longer CS

IH_COMMUNITY_CHAN_CURR_ID_CS             = 1
IH_OUTREACH_CHAN_CURR_ID_CS              = 2
IH_CLINICAL_CURR_ID_CS                   = 3
IH_HOSPITAL_CHAN_CURR_ID_CS              = 4
IH_NUM_DEFAULT_DELIV_CHANS_COSTED_CS     = 4 # Community, Outreach, Clinic, Hospital (See below for mapping)

IH_NO_CHAN_MST_ID = 0 

IH_COMMUNITY_CHAN_MST_ID              = 1 # Community
IH_OUTREACH_CHAN_MST_ID               = 2 # Now un-DEFUNCT - Outreach; WAS part of General outpatient services
IH_CLINIC_CHAN_MST_ID                 = 3 # DEFUNCT - Clinic; part of General outpatient services now
IH_FIRST_REFERRAL_CHAN_MST_ID         = 4 # Hospital --> First referral
IH_PREHOSPITAL_EMERGENCY_CHAN_MST_ID  = 5 # Prehospital emergency 
IH_GEN_OUTPATIENT_SERV_CHAN_MST_ID    = 6 # General outpatient services (Clinic gets rolled into this)
IH_SEC_REFERRAL_CHAN_MST_ID           = 7 # Second referral and above

IH_MAX_DELIV_CHANS_COSTED = 15

# Default delivery channel package current IDs. 

# Community 
UH_FamPrevWASHServ  = 1
UH_FamNeonatalCare  = 2
UH_InfantChildFeed  = 3
UH_CommIllnessManag = 4

# Outreach 
UH_PrevCareAdolAdul  = 1
UH_PrevPregnancyCare = 2
UH_HIVAIDSPrevCare   = 3
UH_PrevInfChildCare  = 4

# Clinic 
UH_MatNeonCareClinic  = 1
UH_ManagIllnessClinic = 2

# Hospital 
UH_ClinicFirstRefCare = 1
UH_ClinicSecRefCare   = 2

# Constant for the maximum number of packages each delivery channel can
# have. 
UH_MaxNumPacksPerDelivChan = 4

# Constant for whether to consider costed delivery channels, non-costed
# delivery channels, or both. 
UH_CostedDelivChans    = 1
UH_NonCostedDelivChans = 2
UH_AllDelivChans       = 3

# Constants for the various levels available in delivery channel mode. 
UH_NatProgramLev = 1
UH_DelivChanLev  = 2
UH_PackageLev    = 3
UH_NumDCMLevels  = 3

# Display Summary Costs constants for Costing Type 
UH_ActualCosts      = 0
UH_IncrementalCosts = 1

#  Row for 'Actual costs'/'Planned expenditures' in the UH Summary Costs
# indicator table. 
UH_TotSumCosts = 50

# Maximum number of facility types the user can enter. 
UH_MaxFacTypes = 20

# Maximum number of program areas the user can enter. 
IH_MAX_PROG_AREAS  = 50

# Maximum number of subgroups allowed in a given program area 
IH_MAX_SUBGROUPS = 20

# Below are the master program area IDs. The master ID for a particular
# program area will never change and should correspond to the master IDs for
# the groups in GroupDB.csv. These master IDs should be used to identify
# program areas in all databases. 
IH_ALL_PROG_AREAS_MST_ID          = 0
IH_MATERNAL_NEWBORN_HEALTH_MST_ID = 1
IH_SEXUAL_REPROD_HEALTH_MST_ID    = 111
UH_ChildHealthMstID               = 15
UH_VaccinationMstID               = 20
UH_MalariaMstID                   = 21
IH_TB_MST_ID                      = 22
IH_HIV_AIDS_MST_ID                = 23
UH_NutritionMstID                 = 28
UH_WASHMstID                      = 29
IH_NCD_MST_ID                     = 30 # NCD = Non-communicable diseases
UH_MentalNueroSubsUseDisMstID     = 42
UH_AdolescentHealthMstID          = 73
IH_NTD_MST_ID                     = 74 # NTD = Neglected tropical diseases
UH_NumDefProgramAreas             = 13

IH_LC_PERICONCEPTUAL_MST_ID = 1
IH_LC_PREGNANCY_MST_ID = 4
IH_LC_CHILDBIRTH_MST_ID = 7
IH_LC_BREASTFEEDING_MST_ID = 13
IH_LC_POSTNATAL_PREV_MST_ID = 14
IH_LC_VACCINES_MST_ID = 15
IH_LC_POSTNATAL_CURATIVE_MST_ID = 16

IH_LC_PROG_AREA_MST_IDS = [IH_LC_PERICONCEPTUAL_MST_ID, IH_LC_PREGNANCY_MST_ID, IH_LC_CHILDBIRTH_MST_ID, IH_LC_BREASTFEEDING_MST_ID,
  IH_LC_POSTNATAL_PREV_MST_ID, IH_LC_VACCINES_MST_ID, IH_LC_POSTNATAL_CURATIVE_MST_ID]

# TIME Econ - For each active pathway, if using target populations from DAT,
# we create a custom intervention and assign it to a special programme area in
# TIME Econ. Default interventions belong to a different group structure
# than the programme areas. It's the same group structure LiST Costing uses
# (from the IV module).
# 
#TIME Econ will also have a custom programme area for custom interventions. 
UH_TE_ActivePathProgArea    = 1
UH_TE_CustomIntervsProgArea = 2

# Special programme areas that are not created by default but need a
# master ID so that it can be referred to specifically. 
UH_HPVMstID                 = 31
UH_WorkersHealthMstID       = 32
UH_ActiveCaseFindingMstID   = 33

#HPV transfers
UH_manufactToCentral = 1
UH_CentralToInterm   = 2
UH_IntermToFacility  = 3
UH_MaxHPVTransfer    = 3

UH_DefHPVCost = 500000

UH_Age15To44 = 1
UH_Age45To54 = 2
UH_Age55To64 = 3
UH_Age65ToXX = 4
UH_NumHPVAges = 4

UH_MaxHPVOutReach = 4

UH_Age15 = 15
UH_Age45 = 45
UH_Age55 = 55
UH_Age65 = 65
UH_Age75 = 75

UH_HPVUnDiscount = 1
UH_HPVDiscount   = 2
UH_HPVLifesYears = 2

UH_HPVstartAge = 15
UH_HPVEndAge   = 75

UH_HPVPreVaccine = 1
UH_HPVPostVaccine = 2
UH_HPVDiffVaccine = 3
UH_HPVNumVaccineDisp = 2

# intervention group IDS for TB, may be removed later when dynamic groups are created
# constants come from groupDB.csv file
UH_mstNotificationSubGrp = 31
UH_mstMDRNotiSubGrp      = 32
UH_mstCollabTBHIVSubGrp  = 33

# Year constants 
UH_FirstYear = 1
UH_FinalYear = 2

#IC Drugs/Supplies 
UH_malariaSupplies        = 1
UH_ChildHealthSupplies    = 3
UH_ContraceptiveSupplies  = 5
UH_EPIPM                  = 6
UH_EquipmentSupplies      = 7
UH_HIVCareTreatment       = 11
UH_ITNs                   = 18
UH_MaternalHealthSupplies = 20
UH_NutritionSupplies      = 23
UH_TBSupplies             = 28
UH_VaccineSupplies        = 31
UH_AllOtherSupplies       = 34

# Constants indicating which data set to use (default or user-entered
# data)
UH_Default = 1
UH_Data    = 2

#*************************************************************************
#***************************   Area constants   **************************
#*************************************************************************

# Area type constants 
IH_PROG_AREA_MST_ID                 = 1
IH_DELIV_CHAN_MST_ID                = 2
IH_DELIV_CHAN_PACK_MST_ID           = 3
IH_NAT_PROG_MST_ID                  = 4
IH_HEALTH_SYS_MST_ID                = 5
IH_TOOL_MST_ID                      = 6
IH_MODULE_MST_ID                    = 7
IH_PPLI_MST_ID                      = 8
IH_SOCIAL_ENABLERS_MST_ID           = 9

# Health Systems / Programme Management (HSPM) constants 
IH_HEALTH_WORKFORCE_HSPM_MST_ID      = 1
IH_INFRASTRUCTURE_HSPM_MST_ID        = 2
IH_LOGISTICS_HSPM_MST_ID             = 3
IH_HEALTH_FINANCING_HSPM_MST_ID      = 4
IH_HEALTH_INFORM_SYSTEMS_HSPM_MST_ID = 5
IH_GOVERNANCE_HSPM_MST_ID            = 6
IH_FINANCIAL_SPACE_HSPM_MST_ID       = 7
IH_NUM_HEALTH_SYSTEMS                = 7

# Audit item (AI) constants 
# Health services
UH_AI_DirectEntryTargPop0Intervs      = 1
UH_AI_FirstAuditItem                  = UH_AI_DirectEntryTargPop0Intervs
UH_AI_FirstHealthServicesItem         = UH_AI_DirectEntryTargPop0Intervs
UH_AI_IntervsNoTreatInputs            = 2
UH_AI_DrugSupplyUsedCostZero          = 3
UH_AI_PercDelivTreatInputMismatch     = 4
UH_AI_CostPerAvgCaseZero              = 5
UH_AI_IntervsScaledDownCoverage       = 6
UH_AI_IntervsConstantCoverage         = 7
UH_AI_ModernFPMethodsHighCoverage     = 8
UH_AI_ModernFPMethodsCovIncrTooFast   = 9
UH_AI_Adults2ndLineARTHighCoverage    = 10
UH_AI_PCAreaTotalsNonZero             = 11
UH_AI_HealthServicesCostTooHigh       = 12
UH_AI_FinalHealthServicesItem         = UH_AI_HealthServicesCostTooHigh
# Logistics 
UH_AI_WastageFilledIn                 = 13
UH_AI_FirstLGItem                     = UH_AI_WastageFilledIn
UH_AI_DrugsSuppliesHighCost           = 14
UH_AI_FinalLGItem                     = UH_AI_DrugsSuppliesHighCost
# Human Resources 
UH_AI_TotalCompensTooHigh             = 15
UH_AI_FirstHRItem                     = UH_AI_TotalCompensTooHigh
UH_AI_StaffTimeUtilizFilledIn         = 16
UH_AI_StaffCapUtilizTooHigh           = 17
UH_AI_KeyStaffCritThresholdReached    = 18
UH_AI_StaffTotalScaledDown            = 19
UH_AI_FinalHRItem                     = UH_AI_StaffTotalScaledDown
# Infrastructure 
UH_AI_FacilNoOpCosts                  = 20
UH_AI_FirstISItem                     = UH_AI_FacilNoOpCosts
UH_AI_FacilBuildEquipOpCostsTooHigh   = 21
UH_AI_FinalISItem                     = UH_AI_FacilBuildEquipOpCostsTooHigh
# Health Information Systems 
UH_AI_HSCostsNonZero                  = 22
UH_AI_FirstHSItem                     = UH_AI_HSCostsNonZero
UH_AI_HSCostsTooHigh                  = 23
UH_AI_FinalHSItem                     = UH_AI_HSCostsTooHigh
# Health Financing 
UH_AI_HFCostsNonZero                  = 24
UH_AI_FirstHFItem                     = UH_AI_HFCostsNonZero
UH_AI_HFCostsTooHigh                  = 25
UH_AI_FinalHFItem                     = UH_AI_HFCostsTooHigh
# Governance 
UH_AI_GVCostsNonZero                  = 26
UH_AI_FirstGVItem                     = UH_AI_GVCostsNonZero
UH_AI_GetGVCostsTooHigh               = 27
UH_AI_FinalGVItem                     = UH_AI_GetGVCostsTooHigh
UH_AI_NumAuditItems                   = UH_AI_GetGVCostsTooHigh

# Set of all audit items that are the first items under their headings. 
UH_AI_FirstItems = [UH_AI_FirstHealthServicesItem, UH_AI_FirstLGItem,
  UH_AI_FirstHRItem, UH_AI_FirstISItem, UH_AI_FirstHSItem, UH_AI_FirstHFItem,
  UH_AI_FirstGVItem]

# Set of all audit items allowing the user to click a button to view
# details AND FIX the problem. 
UH_AI_DetailsAvailableSet = [UH_AI_DirectEntryTargPop0Intervs,
  UH_AI_IntervsNoTreatInputs, UH_AI_DrugSupplyUsedCostZero,
  UH_AI_PercDelivTreatInputMismatch, UH_AI_CostPerAvgCaseZero,
  UH_AI_IntervsScaledDownCoverage, UH_AI_IntervsConstantCoverage,
  UH_AI_StaffTimeUtilizFilledIn, UH_AI_StaffCapUtilizTooHigh,
  UH_AI_PCAreaTotalsNonZero]

# Set of all audit items allowing the user to click a button to DIRECTLY
# FIX them without going to an intermediate form to view details first. 
UH_AI_FixDirectLinkSet = [UH_AI_ModernFPMethodsHighCoverage,
  UH_AI_ModernFPMethodsCovIncrTooFast, UH_AI_Adults2ndLineARTHighCoverage,
  UH_AI_WastageFilledIn]

# Set of all audit items allowing the user to click a button to view details
# BUT NOT FIX the problems. 
UH_AI_ShowDetailsNoFixSet = [UH_AI_DrugsSuppliesHighCost,
  UH_AI_TotalCompensTooHigh, UH_AI_StaffTotalScaledDown,
  UH_AI_FacilNoOpCosts, UH_AI_FacilBuildEquipOpCostsTooHigh,
  UH_AI_StaffCapUtilizTooHigh]

# All audit items that list interventions. 

UH_AI_InterventionsSet = [UH_AI_DirectEntryTargPop0Intervs,
  UH_AI_IntervsNoTreatInputs, UH_AI_PercDelivTreatInputMismatch,
  UH_AI_IntervsScaledDownCoverage, UH_AI_IntervsConstantCoverage]

# All audit items that list custom staff types. 

UH_AI_StaffTypesSet = [UH_AI_TotalCompensTooHigh,
  UH_AI_StaffTimeUtilizFilledIn, UH_AI_StaffTotalScaledDown]

# All audit items that list drugs and supplies. 

UH_AI_DrugSupplySet = [UH_AI_DrugSupplyUsedCostZero,
  UH_AI_DrugsSuppliesHighCost]

# All audit items that list facilities. 

UH_AI_FacilitiesSet = [UH_AI_FacilNoOpCosts,
  UH_AI_FacilBuildEquipOpCostsTooHigh]

UH_StandardTotalCosts = 1
UH_PHC_TotalCosts     = 2
UH_TotalCostTypes     = UH_PHC_TotalCosts

# Custom intervention types
# 
#      IHT :
#       Standard and active pathway. Both types managed by UH.
# 
#      TIME Econ :
#        Standard and active pathway. Standard managed by IV, active pathway
#        managed by UH.
# 
#      LiST Costing :
#        Standard. Managed by IV.

UH_AnyCustomInterv           = 1
UH_StdCustomInterv           = 2
UH_ActivePathwayCustomInterv = 3

IH_INTERVENTION_DB_NAME = ''
IH_GROUP_DB_NAME        = ''

IH_COMPREHENSIVE_PLAN = 0
IH_SPECIFIC_PLAN      = 1

# Staff capacity utilization result is number of programme areas + these 3 things long in the first dimension.
IH_SCU_TRAINING                  = 1
IH_SCU_TASKS_OTHER_THAN_TRAINING = 2
IH_SCU_FTE_AVAILABLE             = 3
IH_STAFF_CAP_UTIL_EXTRAS         = 3

# Health system administration unit (HSAU) defaults

IH_NATIONAL_HSAU_MST_ID         = 1 
IH_PROVINCE_HSAU_MST_ID         = 2
IH_DISTRICT_HSAU_MST_ID         = 3
IH_ADMIN_LEVEL_4_HSAU_MST_ID    = 4 
IH_ADMIN_LEVEL_5_HSAU_MST_ID    = 5 


from os import environ

from AvenirCommon.Database import GB_get_db_json

from SpectrumCommon.Const.GB import GB_SPECT_MOD_DATA_CONN_ENV
from SpectrumCommon.Const.AM import AM_IncidenceByFitTag, AM_SubnatPrevalenceTag, AM_SubnatARTCoverageTag, AM_SubnatCustomBaseYrFlagTag

def am_apply_naomi_subnat(projection, subnat_region):
    
    subnat = GB_get_db_json(environ[GB_SPECT_MOD_DATA_CONN_ENV], 'demproj', f'subnationals\\{subnat_region}_V1.JSON')
    
    projection[AM_IncidenceByFitTag][0] =[subnat['totalIncidence'] for v in projection[AM_IncidenceByFitTag][0]]
    projection[AM_SubnatPrevalenceTag] = subnat['prevalence']
    projection[AM_SubnatARTCoverageTag] = subnat['art_coverage']
    projection[AM_SubnatCustomBaseYrFlagTag] = True
    
    return projection


# Contains methods for setting up more complicated IHT ModVars; others can just be set up in ICModVars.py

import numpy as np

import SpectrumCommon.Const.GB.GBConst as gbc

import SpectrumCommon.Const.IC.ICConst as icc
import SpectrumCommon.Const.IC.ICTargetPopulationIDs as ictpi
import SpectrumCommon.Const.IC.ICConstLink as iccl
import SpectrumCommon.Const.IC.ICModVarFields as icmvf

#######################################################################################################
#                                                                                                     #
#                                       Target populations                                            #
#                                                                                                     #
#######################################################################################################


# These targ pop dicts are attached to interventions
def IC_create_interv_targ_pop_rec(num_years: int):

    return {
        icmvf.IC_TARG_POP_MST_ID: ictpi.IC_TP_TOTAL_POPULATION_MST_ID,
        icmvf.IC_TARG_POP_SEX: gbc.GB_Male,
        icmvf.IC_TARG_POP_START_AGE: 0,
        icmvf.IC_TARG_POP_END_AGE: 80,
        icmvf.IC_TARG_POP_DIRECT_ENTRY_VALUES: np.full((num_years), 0.0).tolist(),
    }


# These targ pop dicts define each target population and are not attached / assigned directly to interventions.
def IC_create_targ_pop_def_rec(name: str, string_constant: str, mst_ID: int):

    return {
        icmvf.IC_TARG_POP_DEF_NAME: name,
        icmvf.IC_TARG_POP_DEF_STR_CONST: string_constant,
        icmvf.IC_TARG_POP_DEF_MST_ID: mst_ID,
        icmvf.IC_TARG_POP_DEF_APPLIC_PROG_AREA_IDS: [],
        icmvf.IC_TARG_POP_DEF_REQ_MOD_MST_IDS: [],
    }


#######################################################################################################
#                                                                                                     #
#                                       Interventions                                                 #
#                                                                                                     #
#######################################################################################################


def IC_create_interv_rec(
    mst_ID: str,
    name: str,
    string_const: str,
    active: bool,
    impact: bool,
    prog_area_ID: str,
    subgroup_ID: str,
    num_deliv_channels: int,
    num_years: int,
):

    return {
        icmvf.IC_INTERV_ID: mst_ID,
        icmvf.IC_INTERV_NAME: name,
        icmvf.IC_INTERV_STR_CONST: string_const,
        icmvf.IC_INTERV_ACTIVE: active,
        icmvf.IC_INTERV_CUSTOM: False,
        icmvf.IC_INTERV_IMPACT: impact,
        icmvf.IC_INTERV_PROG_AREA_ID: prog_area_ID,
        icmvf.IC_INTERV_SUBGROUP_ID: subgroup_ID,
        icmvf.IC_INTERV_SPECIAL_CASES: [],
        icmvf.IC_INTERV_STRATIFYING: False,
        icmvf.IC_INTERV_COST_FOR_PHC: False,
        icmvf.IC_INTERV_PIN_EDITOR_VISITED: False,
        icmvf.IC_INTERV_COVERAGE_CHANGED: False,
        icmvf.IC_INTERV_PIN_STR_CONSTANT: "",
        icmvf.IC_INTERV_COVERAGE: np.full((num_years), 0.0).tolist(),
        icmvf.IC_INTERV_TREAT_INPUT_NOT_REQ: [],
        icmvf.IC_INTERV_MAX_CUSTOM_TI_GROUP_MST_ID: [],
        icmvf.IC_INTERV_PIN: np.full((num_years), 0.0).tolist(),
        icmvf.IC_INTERV_PERC_DELIV_THROUGH_CHAN: np.full((num_deliv_channels, icc.IC_NUM_YEAR_ENDPOINTS), 0.0).tolist(),
        icmvf.IC_INTERV_VISIT_COST_PER_CASE: [],
        icmvf.IC_INTERV_TARG_POP_REC: IC_create_interv_targ_pop_rec(num_years),
    }


# This creates an intervention record with only enough info in it to identify and display it in a list.
def IC_create_interv_rec_ID_info_only(mst_ID: str, name: str, string_const: str, prog_area_ID: str, subgroup_ID=str):

    return {
        icmvf.IC_INTERV_ID: mst_ID,
        icmvf.IC_INTERV_NAME: name,
        icmvf.IC_INTERV_STR_CONST: string_const,
        icmvf.IC_INTERV_PROG_AREA_ID: prog_area_ID,
        icmvf.IC_INTERV_SUBGROUP_ID: subgroup_ID,
    }


#######################################################################################################
#                                                                                                     #
#                                  Alternative coverage records                                       #
#                                                                                                     #
#######################################################################################################


def IC_create_alt_cov_dict(interv_ID: str, num_years: int):

    return {
        icmvf.IC_ALT_COV_INTERV_ID: interv_ID,
        icmvf.IC_ALT_COV_VALUE: np.full((num_years), 0.0).tolist(),
    }


#######################################################################################################
#                                                                                                     #
#                                  Alternative PIN age group set 1 records                            #
#                                                                                                     #
#######################################################################################################


def IC_create_alt_PIN_age_group_set_1_rec(interv_ID: str, num_years: int):

    return {
        icmvf.IC_ALT_PIN_AGE_GROUP_SET_1_INTERV_ID: interv_ID,
        icmvf.IC_ALT_PIN_AGE_GROUP_SET_1_VALUE: np.full((icc.IC_NUM_AGE_GROUPS_SET_1, num_years), 0.0).tolist(),
    }


#######################################################################################################
#                                                                                                     #
#                                  Alternative PIN age group set 2 records                            #
#                                                                                                     #
#######################################################################################################


def IC_create_alt_PIN_age_group_set_2_rec(interv_ID: str, num_years: int):

    return {
        icmvf.IC_ALT_PIN_AGE_GROUP_SET_2_INTERV_ID: interv_ID,
        icmvf.IC_ALT_PIN_AGE_GROUP_SET_2_VALUE: np.full((icc.IC_NUM_AGE_GROUPS_SET_2, num_years), 0.0).tolist(),
    }


#######################################################################################################
#                                                                                                     #
#                                       Drugs and supplies                                            #
#                                                                                                     #
#######################################################################################################


def IC_create_drug_supply_rec(
    ID: str,
    name: str,
    active: bool,
    cost_type_LG_mstID: int,
    drug_supply_type_mst_ID: int,
    requires_cold_storage: bool,
    num_years: int,
    costing_mode: int,
    plan_type: int,
):
    drug_supply_rec = {
        icmvf.IC_DRUG_SUPPLY_ID: ID,
        icmvf.IC_DRUG_SUPPLY_NAME: name,
        icmvf.IC_DRUG_SUPPLY_STR_CONST: "",
        icmvf.IC_DRUG_SUPPLY_COST_TYPE_LG_MST_ID: cost_type_LG_mstID,  # Used by the Medicines, commodities, and supplies Summary result.
        icmvf.IC_DRUG_SUPPLY_ACTIVE: active,
        icmvf.IC_DRUG_SUPPLY_CUSTOM: False,
        icmvf.IC_DRUG_SUPPLY_TYPE_MST_ID: drug_supply_type_mst_ID,
        icmvf.IC_DRUG_SUPPLY_APPLIC_PROG_AREA_LIST: [],
        icmvf.IC_DRUG_SUPPLY_UNIT_COST: np.full((num_years), 0.0).tolist(),
        # Used in Logistics
        icmvf.IC_DRUG_SUPPLY_PERC_WASTAGE: 0,
        icmvf.IC_DRUG_SUPPLY_PERC_SAFETY_STOCK_NEEDED: 0,
        icmvf.IC_DRUG_SUPPLY_SAFETY_STOCK_BASELINE: 0,
        icmvf.IC_DRUG_SUPPLY_PERC_SAFETY_STOCK_REC: np.full((num_years), 0.0).tolist(),
        icmvf.IC_DRUG_SUPPLY_QUANTITY: np.full((num_years), 0.0).tolist(),
        icmvf.IC_DRUG_SUPPLY_UNIT_WEIGHT: 0.0,
        icmvf.IC_DRUG_SUPPLY_UNIT_VOLUME: 0.0,
        icmvf.IC_DRUG_SUPPLY_REQUIRES_COLD_STORAGE: requires_cold_storage,
    }

    if ((costing_mode == gbc.GB_IHT_COSTING_MODE) and (plan_type == iccl.IC_IH_SPECIFIC_PLAN)) or (
        costing_mode == gbc.GB_TB_COSTING_MODE
    ):
        drug_supply_rec[icmvf.IC_DRUG_SUPPLY_LOG_SUPP_CHAIN_COSTS_PERC] = 0
        drug_supply_rec[icmvf.IC_DRUG_SUPPLY_WASTAGE_PERC] = 0

    return drug_supply_rec


#######################################################################################################
#                                                                                                     #
#                                       Treatment input group (TIG) records                           #
#                                                                                                     #
#######################################################################################################


def IC_create_treat_input_group_rec(interv_ID: str, deliv_chan_ID: str, treat_input_group_ID: str, name=str):

    return {
        icmvf.IC_TIG_INTERV_ID: interv_ID,
        icmvf.IC_TIG_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_TIG_ID: treat_input_group_ID,
        icmvf.IC_TIG_NAME: name,
        icmvf.IC_TIG_STR_CONST: "",
    }


#######################################################################################################
#                                                                                                     #
#                                       Treatment input (TI) base records                             #
#                                                                                                     #
#######################################################################################################


def IC_create_treat_input_rec(
    interv_ID: str,
    deliv_chan_ID: str,
    treat_input_type_mst_ID: int,
    treat_input_group_ID: str,
    treat_input_ID: str,
    num_years: int,
):

    TI_rec = {
        icmvf.IC_TI_INTERV_ID: interv_ID,
        icmvf.IC_TI_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_TI_GROUP_ID: treat_input_group_ID,
        icmvf.IC_TI_ID: treat_input_ID,
        icmvf.IC_TI_PERC_APPLIED: np.full((num_years), 0.0).tolist(),
        icmvf.IC_TI_NOTE: "",
        icmvf.IC_TI_NOTE_STR_CONST: "",
    }

    if treat_input_type_mst_ID == icc.IC_TI_DRUGS_SUPPLIES_MST_ID:
        TI_rec[icmvf.IC_TI_DRUG_SUPPLY_TIMES_PER_DAY] = 0
        TI_rec[icmvf.IC_TI_DRUG_SUPPLY_DAYS_PER_CASE] = 0
        TI_rec[icmvf.IC_TI_DRUG_SUPPLY_NUMBER] = 0

    elif treat_input_type_mst_ID == icc.IC_TI_OUTPAT_VISITS_INPAT_DAYS_MST_ID:
        TI_rec[icmvf.IC_TI_OUTPAT_INPAT_UNITS_PER_CASE] = 0.0

    elif treat_input_type_mst_ID == icc.IC_TI_MED_PERSONNEL_TIME_MST_ID:
        TI_rec[icmvf.IC_TI_MED_PERSONNEL_CUSTOM_STAFF_TYPE] = False
        TI_rec[icmvf.IC_TI_MED_PERSONNEL_SKILLED_STAFF_COST] = False
        TI_rec[icmvf.IC_TI_MED_PERSONNEL_NUM_OF_DAYS] = 0
        TI_rec[icmvf.IC_TI_MED_PERSONNEL_MINUTES] = 0.0

    return TI_rec


#######################################################################################################
#                                                                                                     #
#                                       Cost per case records                                         #
#                                                                                                     #
#######################################################################################################


def IC_create_cost_per_case_rec(interv_ID: str, deliv_chan_ID: str, value=list):

    return {
        icmvf.IC_F_INTERV_ID: interv_ID,
        icmvf.IC_F_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_F_VALUE: value,
    }


#######################################################################################################
#                                                                                                     #
#                             Minutes per average case (default staff) records                        #
#                                                                                                     #
#######################################################################################################


def IC_create_min_per_avg_case_static_staff_rec(
    interv_ID: str, deliv_chan_ID: str, static_staff_mst_ID: int, value=list
):

    return {
        icmvf.IC_F_INTERV_ID: interv_ID,
        icmvf.IC_F_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_F_STATIC_STAFF_MST_ID: static_staff_mst_ID,
        icmvf.IC_F_VALUE: value,
    }


#######################################################################################################
#                                                                                                     #
#                             Minutes per average case (custom staff) records                         #
#                                                                                                     #
#######################################################################################################


def IC_create_min_per_avg_case_custom_staff_rec(
    interv_ID: str, deliv_chan_ID: str, custom_staff_mst_ID: int, value=list
):

    return {
        icmvf.IC_F_INTERV_ID: interv_ID,
        icmvf.IC_F_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_F_DYNAMIC_STAFF_MST_ID: custom_staff_mst_ID,
        icmvf.IC_F_VALUE: value,
    }


#######################################################################################################
#                                                                                                     #
#                             Inpatient days per average case records                                 #
#                                                                                                     #
#######################################################################################################


def IC_create_inpatient_days_outpatient_visits_per_avg_case_rec(
    interv_ID: str, deliv_chan_ID: str, treat_input_ID: int, value: list
):

    return {
        icmvf.IC_F_INTERV_ID: interv_ID,
        icmvf.IC_F_DELIV_CHAN_ID: deliv_chan_ID,
        icmvf.IC_F_TREAT_INPUT_ID: treat_input_ID,
        icmvf.IC_F_VALUE: value,
    }


#######################################################################################################
#                                                                                                     #
#                                            Results                                                  #
#                                                                                                     #
#######################################################################################################


def IC_init_interv_costed_deliv_chan_year_arr(num_intervs: int, num_costed_deliv_chans: int, num_years: int, value=0.0):
    return np.full((num_intervs, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_deliv_chan_all_year_arr(num_intervs: int, num_deliv_chans_all: int, num_years: int, value=0.0):
    return np.full((num_intervs, num_deliv_chans_all, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_drug_supply_deliv_chan_all_year_arr(
    num_drugs_supplies: int, num_deliv_chans_all: int, num_years: int, value=0.0
):
    return np.full((num_drugs_supplies, num_deliv_chans_all, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_staff_costed_deliv_chan_year_arr(
    num_intervs: int, num_staff_types: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full((num_intervs, num_staff_types, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_static_staff_costed_deliv_chan_year_arr(
    num_intervs: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full(
        (num_intervs, iccl.IC_HW_NUM_STATIC_STAFF_TYPES, num_costed_deliv_chans, num_years),
        value,
        gbc.GB_CALC_FLOAT_TYPE,
    )


def IC_init_interv_year_arr(num_intervs: int, num_years: int, value=0.0):
    return np.full((num_intervs, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_arr(num_intervs: int, value=0.0):
    return np.full((num_intervs), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_empty_arr(value=0):
    return np.empty(value)


def IC_init_year_arr(num_years: int, value=0.0):
    return np.full((num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_costed_deliv_chan_year_arr(num_costed_deliv_chans: int, num_years: int, value=0.0):
    return np.full((num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_static_staff_costed_deliv_chan_year_arr(num_costed_deliv_chans: int, num_years: int, value=0.0):
    return np.full(
        (iccl.IC_HW_NUM_STATIC_STAFF_TYPES, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE
    )


def IC_init_static_staff_interv_year_arr(num_intervs: int, num_years: int, value=0.0):
    return np.full((iccl.IC_HW_NUM_STATIC_STAFF_TYPES, num_intervs, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_static_staff_prog_area_year_arr(num_prog_areas: int, num_years: int, value=0.0):
    return np.full((iccl.IC_HW_NUM_STATIC_STAFF_TYPES, num_prog_areas, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_static_staff_prog_area_deliv_chan_year_arr(
    num_prog_areas: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full(
        (iccl.IC_HW_NUM_STATIC_STAFF_TYPES, num_prog_areas, num_costed_deliv_chans, num_years),
        value,
        gbc.GB_CALC_FLOAT_TYPE,
    )


def IC_init_dynamic_staff_costed_deliv_chan_year_arr(
    num_dynamic_staff_types: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full((num_dynamic_staff_types, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_dynamic_staff_interv_year_arr(num_dynamic_staff_types, num_intervs: int, num_years: int, value=0.0):
    return np.full((num_dynamic_staff_types, num_intervs, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_dynamic_staff_prog_area_year_arr(
    num_dynamic_staff_types: int, num_prog_areas: int, num_years: int, value=0.0
):
    return np.full((num_dynamic_staff_types, num_prog_areas, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_dynamic_staff_prog_area_deliv_chan_year_arr(
    num_dynamic_staff_types: int, num_prog_areas: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full(
        (num_dynamic_staff_types, num_prog_areas, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE
    )


def IC_init_drug_supply_year_arr(num_drugs_supplies: int, num_years: int, value=0.0):
    return np.full((num_drugs_supplies, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_drug_supply_interv_costed_deliv_chan_year_arr(
    num_drugs_supplies: int, num_intervs: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full((num_drugs_supplies, num_intervs, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_drug_supply_costed_deliv_chan_year_arr(
    num_drugs_supplies: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full((num_drugs_supplies, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_drug_supply_prog_area_year_arr(num_drugs_supplies: int, num_prog_areas: int, num_years: int, value=0.0):
    return np.full((num_drugs_supplies, num_prog_areas, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_visit_costed_deliv_chan_year_arr(
    num_intervs: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full(
        (num_intervs, icc.IC_NUM_VISIT_TYPES, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE
    )


def IC_init_visit_costed_deliv_chan_arr(num_costed_deliv_chans: int, value=0.0):
    return np.full((icc.IC_NUM_VISIT_TYPES, num_costed_deliv_chans), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_costing_sect_costed_deliv_chan_year_arr(
    num_intervs: int, num_costing_sect: int, num_costed_deliv_chans: int, num_years: int, value=0.0
):
    return np.full((num_intervs, num_costing_sect, num_costed_deliv_chans, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_interv_costing_sect_year_arr(num_intervs: int, num_costing_sect: int, num_years: int, value=0.0):
    return np.full((num_intervs, num_costing_sect, num_years), value, gbc.GB_CALC_FLOAT_TYPE)


def IC_init_costing_sect_year_arr(num_costing_sect: int, num_years: int, value=0.0):
    return np.full((num_costing_sect, num_years), value, gbc.GB_CALC_FLOAT_TYPE)



IV_GROUP_TYPE = 1
IV_SUBGROUP_TYPE = 2

# Database latest versions. We need to have different versions for each app or else we would have to upload
# all databases each time they change.
IV_IH_INTERVENTION_DB_CURR_VERSION = "V26"
IV_IH_GROUP_DB_CURR_VERSION = "V7"

IV_TI_INTERVENTION_DB_CURR_VERSION = "V18"
IV_TI_GROUP_DB_CURR_VERSION = "V5"

IV_CS_INTERVENTION_DB_CURR_VERSION = "V22"
IV_CS_GROUP_DB_CURR_VERSION = "V2"

# Database names (also the names of the sheets in IVModData.xlsx)
IV_INTERVENTION_DB_NAME = "InterventionDB"
IV_GROUP_DB_NAME = "GroupDB"



from os import environ

import SpectrumCommon.Const.GB as gbc

from AvenirCommon.Database import GB_get_db_json
import AvenirCommon.Util as acu

import SpectrumCommon.Const.PC.PCConst as pcc
import SpectrumCommon.Const.PC.PCConstLink as pccl
import SpectrumCommon.Const.PC.PCDatabaseKeys as pcdbk

import SpectrumCommon.Const.PC.PCModVarFields as pcmvf
import SpectrumCommon.Modvars.PC.PCModVarDefs as pcmvd
import SpectrumCommon.Modvars.PC.PCModVarUtil as pcmvu
import SpectrumCommon.Modvars.PC.PCUtil as pcu

#######################################################################################################
#                                                                                                     #
#                                       Database loading methods                                      #
#                                                                                                     #
#######################################################################################################

def PC_load_cost_cat_DB():
    cost_cat_DB_file_name = pcc.PC_COST_CAT_DB_NAME + '_' + pcc.PC_COST_CAT_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    cost_cat_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, cost_cat_DB_file_name) 

    return cost_cat_DB

def PC_load_cost_input_DB():
    cost_input_DB_file_name = pcc.PC_COST_INPUT_DB_NAME + '_' + pcc.PC_COST_INPUT_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    cost_input_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, cost_input_DB_file_name) 

    return cost_input_DB

def PC_load_activity_DB():
    activity_DB_file_name = pcc.PC_ACTIVITY_DB_NAME + '_' + pcc.PC_ACTIVITY_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    activity_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, activity_DB_file_name) 

    return activity_DB

def PC_load_activity_line_item_DB():
    activity_line_item_DB_file_name = pcc.PC_ACTIVITY_LINE_ITEM_DB_NAME + '_' + pcc.PC_ACTIVITY_LINE_ITEM_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    activity_line_item_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, activity_line_item_DB_file_name) 

    return activity_line_item_DB

def PC_load_PPLI_activity_line_item_DB():
    PPLI_activity_line_item_DB_file_name = pcc.PC_PPLI_ACTIVITY_LINE_ITEM_DB_NAME + '_' + pcc.PC_PPLI_ACTIVITY_LINE_ITEM_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    PPLI_activity_line_item_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, PPLI_activity_line_item_DB_file_name) 

    return PPLI_activity_line_item_DB

def PC_load_costing_sect_DB():
    costing_sect_DB_file_name = pcc.PC_COSTING_SECT_DB_NAME + '_' + pcc.PC_COSTING_SECT_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    costing_sect_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, costing_sect_DB_file_name) 

    return costing_sect_DB

def PC_load_PPLI_phase_DB(country_ID):
    PPLI_phase_DB_file_name = pcc.PC_PPLI_PHASES_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + pcc.PC_PPLI_PHASE_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    PPLI_phase_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, PPLI_phase_DB_file_name) 

    return PPLI_phase_DB

def PC_load_PPLI_taxation_DB(country_ID):
    PPLI_taxation_DB_file_name = pcc.PC_PPLI_TAXATION_DB_DIR.replace('\\', '/') + '/' + country_ID + '_' + pcc.PC_PPLI_TAXATION_DB_CURR_VERSION + '.' + gbc.GB_JSON 
    PPLI_taxation_DB = GB_get_db_json(environ[gbc.GB_SPECT_MOD_DATA_CONN_ENV], gbc.GB_PC_CONTAINER, PPLI_taxation_DB_file_name) 

    return PPLI_taxation_DB

#######################################################################################################
#                                                                                                     #
#                                       Cost categories                                               #
#                                                                                                     #
#######################################################################################################

def PC_init_cost_cat_records(cost_cat_DB):
    cost_cat_records = []

    for cost_cat_DB_obj in cost_cat_DB:
        ID = str(cost_cat_DB_obj[pcdbk.PC_COST_CAT_ID_KEY_CCDB])
        name = cost_cat_DB_obj[pcdbk.PC_COST_CAT_NAME_KEY_CCDB] 
        string_constant = cost_cat_DB_obj[pcdbk.PC_COST_CAT_STR_CONST_KEY_CCDB] 

        cost_cat_rec = pcmvd.PC_create_cost_cat_rec(ID, name, string_constant)
        cost_cat_records.append(cost_cat_rec)

    return cost_cat_records

#######################################################################################################
#                                                                                                     #
#                                       Cost inputs                                                   #
#                                                                                                     #
#######################################################################################################

def PC_init_cost_input_records(cost_input_DB):
    cost_input_records = []

    for cost_input_DB_obj in cost_input_DB:
        cost_input_ID = str(cost_input_DB_obj[pcdbk.PC_COST_INPUT_ID_KEY_CIDB]) 
        name = cost_input_DB_obj[pcdbk.PC_COST_INPUT_NAME_KEY_CIDB] 
        cost_input_string_constant = cost_input_DB_obj[pcdbk.PC_COST_INPUT_STR_CONST_KEY_CIDB] 
        cost_cat_ID = str(cost_input_DB_obj[pcdbk.PC_COST_CAT_ID_KEY_CIDB])
        unit_cost = cost_input_DB_obj[pcdbk.PC_UNIT_COST_KEY_CIDB] 
        unit_of_measure = cost_input_DB_obj[pcdbk.PC_UNIT_OF_MEASURE_KEY_CIDB] 
        unit_of_measure_string_constant = cost_input_DB_obj[pcdbk.PC_UNIT_OF_MEASURE_STR_CONST_KEY_CIDB]
        note = cost_input_DB_obj[pcdbk.PC_NOTE_KEY_CIDB]
        source = cost_input_DB_obj[pcdbk.PC_SOURCE_KEY_CIDB]

        cost_input_rec = pcmvd.PC_create_cost_input_rec(cost_input_ID, name, cost_cat_ID)

        pcmvu.set_PC_cost_input_string_constant(cost_input_rec, cost_input_string_constant)
        pcmvu.set_PC_cost_input_unit_cost(cost_input_rec, unit_cost)
        pcmvu.set_PC_cost_input_unit_of_measure(cost_input_rec, unit_of_measure)
        pcmvu.set_PC_cost_input_unit_of_measure_string_constant(cost_input_rec, unit_of_measure_string_constant)
        pcmvu.set_PC_cost_input_note(cost_input_rec, note)
        pcmvu.set_PC_cost_input_source(cost_input_rec, source)

        cost_input_records.append(cost_input_rec)

    return cost_input_records

def PC_create_rec_from_DB_data(area_type_ID = int, area_ID = int, costing_mode = int, modules = list):
    # Process the DB data if one of the following is true.
    #
    #    1. The area type is modules and one of the following is true
    #       a) the area is the TB module and we're doing IHT TB costing.
    #       b) the area is the CS module and we're doing LiST Costing.
    #       c) the area is the HS module and HS is an active module.
    # 
    #    2. The area type is health systems/programme management, we're doing IHT costing, and
    #       a) the area is health information systems and HS is an active module.
    #       b) the area is infrastructure and IS is an active module.
    #       c) the area is health workforce and HW is an active module.
    #       d) the area is logistics and LG is an active module.
    #
    #    3. The area type is a programme area and we're doing IHT costing.
    #
    #    4. The area type is the special PPLI area in the NCD programme area and we're doing IHT costing.
    # 
    #    5. The area type is the special social enablers area in the HIV programme area and we're doing IHT costing.
    #

    return (
        ((area_type_ID == pccl.PC_IH_MODULE_MST_ID) 
        and (((area_ID == str(gbc.GB_TB)) and (costing_mode == gbc.GB_TB_COSTING_MODE)) 
        or ((area_ID == str(gbc.GB_CS)) and (costing_mode == gbc.GB_LIST_COSTING_MODE))
        or ((area_ID == str(gbc.GB_HS)) and (gbc.GB_HS in modules))))
    
        or
    
        ((area_type_ID == pccl.PC_IH_HEALTH_SYS_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE)
        and (((area_ID == str(pccl.PC_IH_HEALTH_INFORM_SYSTEMS_HSPM_MST_ID)) and (gbc.GB_HS in modules))
        or ((area_ID == str(pccl.PC_IH_INFRASTRUCTURE_HSPM_MST_ID)) and (gbc.GB_IS in modules))
        or ((area_ID == str(pccl.PC_IH_HEALTH_WORKFORCE_HSPM_MST_ID)) and (gbc.GB_HW in modules))
        or ((area_ID == str(pccl.PC_IH_LOGISTICS_HSPM_MST_ID)) and (gbc.GB_LG in modules))))

        or 

        ((area_type_ID == pccl.PC_IH_PROG_AREA_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE))

        or

        ((area_type_ID == pccl.PC_IH_PPLI_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE))

        or

        ((area_type_ID == pccl.PC_IH_SOCIAL_ENABLERS_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE))
    )

#######################################################################################################
#                                                                                                     #
#                         Costing categories and costing category sections                            #
#                                                                                                     #
#######################################################################################################

def PC_init_costing_sect_records(costing_sect_DB, num_years, costing_mode, modules, PPLI):
    costing_sect_records = []

    i = 0

    for costing_sect_DB_obj in costing_sect_DB:
        area_type_ID = costing_sect_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB]
        area_ID = str(costing_sect_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB]) 
        costing_sect_ID = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB]) 
        costing_sect_name = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_NAME_KEY_CSDB])
        costing_sect_string_constant = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_STR_CONST_KEY_CSDB])
        costing_subsect_ID = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]) 
        costing_subsect_name = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_NAME_KEY_CSDB]) 
        costing_subsect_string_constant = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_STR_CONST_KEY_CSDB]) 

        if (not PPLI) and PC_create_rec_from_DB_data(area_type_ID, area_ID, costing_mode, modules):

            # Costing section will have a costing subsection ID of -1. Costing section should always come before their costing subsections in the database.
            if costing_subsect_ID == str(gbc.GB_NOT_FOUND):
                costing_sect_rec = pcmvd.PC_create_costing_section_rec(area_type_ID, area_ID, costing_sect_ID, costing_sect_name, costing_sect_string_constant, num_years, costing_mode)
                costing_sect_records.append(costing_sect_rec)

                if costing_mode == gbc.GB_LIST_COSTING_MODE:
                    for t in acu.GBRange(1, num_years):
                        pcmvu.set_PC_costing_sect_costs_CS(costing_sect_rec, t - 1, costing_sect_DB_obj[pcdbk.PC_COSTS_CS_KEY_CSDB])

            else:
                costing_subsect_rec = pcmvd.PC_create_costing_subsection_rec(costing_subsect_ID, costing_subsect_name, costing_subsect_string_constant)
                '''
                    Since the database lists costing sections first, then their subsections, then repeats, we should always be able to add the subsection to
                    the current section.
                '''
                costing_sect_records[-1][pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS].append(costing_subsect_rec)

            i += 1

        elif PPLI and (area_type_ID == pccl.PC_IH_PPLI_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE):

            # Costing section will have a costing subsection ID of -1. Costing section should always come before their costing subsections in the database.
            if costing_subsect_ID == str(gbc.GB_NOT_FOUND):
                costing_sect_rec = pcmvd.PC_create_costing_section_rec_PPLI(area_ID, costing_sect_ID, costing_sect_name, costing_sect_string_constant, num_years)
                costing_sect_records.append(costing_sect_rec)

            else:
                costing_subsect_rec = pcmvd.PC_create_costing_subsection_rec_PPLI(costing_subsect_ID, costing_subsect_name, costing_subsect_string_constant, num_years)
                '''
                    Since the database lists costing sections first, then their subsections, then repeats, we should always be able to add the subsection to 
                    the current section.
                '''
                costing_sect_records[-1][pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS].append(costing_subsect_rec)
            
    return costing_sect_records

def PC_init_custom_prog_area_costing_sect_records(costing_mode, custom_prog_area_ID):
    # Custom programme areas can only be added in IHT
    if not costing_mode == gbc.GB_IHT_COSTING_MODE: pass

    costing_sect_DB = PC_load_costing_sect_DB()

    custom_DB_records = [costing_sect_DB_obj for costing_sect_DB_obj in costing_sect_DB if costing_sect_DB_obj[pcdbk.PC_AREA_ID_KEY_CSDB] == pcc.PC_COSTING_SECT_DB_AREA_ID_CUSTOM]

    custom_costing_sect_records = []

    for costing_sect_DB_obj in custom_DB_records:
        area_type_ID = costing_sect_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_CSDB]
        area_ID = custom_prog_area_ID
        costing_sect_ID = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_CSDB]) 
        costing_sect_name = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_NAME_KEY_CSDB])
        costing_sect_string_constant = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SECT_STR_CONST_KEY_CSDB])
        costing_subsect_ID = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_CSDB]) 
        costing_subsect_name = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_NAME_KEY_CSDB]) 
        costing_subsect_string_constant = str(costing_sect_DB_obj[pcdbk.PC_COSTING_SUBSECT_STR_CONST_KEY_CSDB]) 

        # Costing section will have a costing subsection ID of -1. Costing section should always come before their costing subsections in the database.
        if costing_subsect_ID == str(gbc.GB_NOT_FOUND):
            costing_sect_rec = pcmvd.PC_create_costing_section_rec(area_type_ID, area_ID, costing_sect_ID, costing_sect_name, costing_sect_string_constant, 0, costing_mode)
            custom_costing_sect_records.append(costing_sect_rec)

        else:
            costing_subsect_rec = pcmvd.PC_create_costing_subsection_rec(costing_subsect_ID, costing_subsect_name, costing_subsect_string_constant)
            # Since the database lists costing sections first, then their subsections, then repeats, we should always be able to add the subsection to the current section.
            custom_costing_sect_records[-1][pcmvf.PC_COSTING_SECT_SUBSECT_RECORDS].append(costing_subsect_rec)
            
    return custom_costing_sect_records

#######################################################################################################
#                                                                                                     #
#                                              Activities                                             #
#                                                                                                     #
#######################################################################################################

def PC_init_activity_records(activity_DB, costing_mode, modules, PPLI):
    activity_records = []

    for activity_DB_obj in activity_DB:
        area_type_ID = activity_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_ADB]
        area_ID = activity_DB_obj[pcdbk.PC_AREA_ID_KEY_ADB]
        costing_sect_ID = str(activity_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_ADB])
        costing_subsect_ID = str(activity_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_ADB])  
        activity_ID = str(activity_DB_obj[pcdbk.PC_ACT_ID_KEY_ADB])     
        name = activity_DB_obj[pcdbk.PC_ACT_NAME_KEY_ADB]   
        string_constant = activity_DB_obj[pcdbk.PC_ACT_STR_CONST_KEY_ADB]   

        if (
            ((not PPLI) and PC_create_rec_from_DB_data(area_type_ID, area_ID, costing_mode, modules)) 
            or (PPLI and (area_type_ID == pccl.PC_IH_PPLI_MST_ID) and (costing_mode == gbc.GB_IHT_COSTING_MODE))
        ):
            activity_rec = pcmvd.PC_create_activity_rec(area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID, activity_ID, name)
            pcmvu.set_PC_act_string_constant(activity_rec, string_constant)
            pcmvu.add_PC_activity_rec(activity_records, activity_rec)
            
    return activity_records

#######################################################################################################
#                                                                                                     #
#                                       Activity line items (ALIs)                                    #
#                                                                                                     #
#######################################################################################################

def PC_init_activity_line_item_records(activity_line_item_DB, num_years, costing_mode, modules):
    ALI_records = []

    for ALI_DB_obj in activity_line_item_DB:
        area_type_ID = ALI_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_ALIDB]
        area_ID = ALI_DB_obj[pcdbk.PC_AREA_ID_KEY_ALIDB]
        costing_sect_ID = ALI_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_ALIDB]
        costing_subsect_ID = ALI_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_ALIDB]
        act_ID = ALI_DB_obj[pcdbk.PC_ACT_ID_KEY_ALIDB]
        cost_input_ID = ALI_DB_obj[pcdbk.PC_COST_INPUT_ID_KEY_ALIDB]
        units = ALI_DB_obj[pcdbk.PC_NUM_UNITS_KEY_ALIDB]
        duration = ALI_DB_obj[pcdbk.PC_DURATION_KEY_ALIDB]
        frequency = ALI_DB_obj[pcdbk.PC_FREQUENCY_KEY_ALIDB]

        if PC_create_rec_from_DB_data(area_type_ID, area_ID, costing_mode, modules):
            ALI_rec = pcmvd.PC_create_activity_line_item_rec(area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID, act_ID, cost_input_ID, num_years, False)

            pcmvu.set_PC_ALI_num_units(ALI_rec, units)
            pcmvu.set_PC_ALI_duration(ALI_rec, duration)

            for t in acu.GBRange(1, num_years):
                pcmvu.set_PC_ALI_frequency(ALI_rec, t - 1, frequency)

            pcmvu.add_PC_ALI_rec(ALI_records, ALI_rec)

    return ALI_records

def PC_init_PPLI_activity_line_item_records(activity_line_item_DB, costing_sect_records, num_years):
    ALI_records = []

    for ALI_DB_obj in activity_line_item_DB:
        area_type_ID = ALI_DB_obj[pcdbk.PC_AREA_TYPE_ID_KEY_PALIDB]
        area_ID = ALI_DB_obj[pcdbk.PC_AREA_ID_KEY_PALIDB]
        costing_sect_ID = ALI_DB_obj[pcdbk.PC_COSTING_SECT_ID_KEY_PALIDB]
        costing_subsect_ID = ALI_DB_obj[pcdbk.PC_COSTING_SUBSECT_ID_KEY_PALIDB]
        act_ID = ALI_DB_obj[pcdbk.PC_ACT_ID_KEY_PALIDB]
        cost_input_ID = ALI_DB_obj[pcdbk.PC_COST_INPUT_ID_KEY_PALIDB]
        units = ALI_DB_obj[pcdbk.PC_NUM_UNITS_KEY_PALIDB]
        duration = ALI_DB_obj[pcdbk.PC_DURATION_KEY_PALIDB]
        quantity = ALI_DB_obj[pcdbk.PC_QUANTITY_KEY_PALIDB]

        costing_subsect_rec = pcmvu.get_PC_costing_subsect_rec(costing_sect_records, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID)

        ALI_rec = pcmvd.PC_create_activity_line_item_rec_PPLI(area_ID, costing_sect_ID, costing_subsect_ID, act_ID, cost_input_ID, num_years)

        for al in acu.GBRange(1, pcc.PC_PPLI_NUM_ADMIN_LEVELS):
            pcmvu.set_PC_ALI_num_units_PPLI(ALI_rec, al, units[al - 1])
            pcmvu.set_PC_ALI_duration_PPLI(ALI_rec, al, duration[al - 1])

            for pas in acu.GBRange(1, pcc.PC_NUM_PPLI_ALI_SETS):
                for t in acu.GBRange(1, num_years):
                    phase_ID = pcmvu.get_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_PHASE_ROW_PPLI_CURR_ID, t - 1)
                    phase_curr_ID = pcu.get_PPLI_phase_curr_ID(phase_ID)

                    pcmvu.set_PC_ALI_quantity(ALI_rec, pas, al, t - 1, quantity[phase_curr_ID - 1][al - 1])

        pcmvu.add_PC_ALI_rec(ALI_records, ALI_rec)

    return ALI_records

#######################################################################################################
#                                                                                                     #
#                                       PPLI phases                                                   #
#                                                                                                     #
#######################################################################################################

def PC_init_PPLI_phase_and_intensity(PPLI_phase_DB, costing_sect_records_PPLI, num_years):
    phase_intensity_records = PPLI_phase_DB[pcdbk.PC_PHASE_INTENSITY_RECORDS_KEY_PPLIPDB]

    for phase_intensity_rec in phase_intensity_records:
        area_type_ID = phase_intensity_rec[pcdbk.PC_AREA_TYPE_ID_KEY_PPLIPDB] 
        area_ID = str(phase_intensity_rec[pcdbk.PC_AREA_ID_KEY_PPLIPDB]) 
        costing_sect_ID = str(phase_intensity_rec[pcdbk.PC_COSTING_SECT_ID_KEY_PPLIPDB])    
        costing_subsect_ID = str(phase_intensity_rec[pcdbk.PC_COSTING_SUBSECT_ID_KEY_PPLIPDB]) 

        costing_subsect_rec = pcmvu.get_PC_costing_subsect_rec(costing_sect_records_PPLI, area_type_ID, area_ID, costing_sect_ID, costing_subsect_ID)

        if costing_subsect_rec != gbc.GB_NOT_FOUND:

            phase_ID = phase_intensity_rec[pcdbk.PC_PHASE_ID_KEY_PPLIPDB]                

            if (
                (area_type_ID == pccl.PC_IH_PPLI_MST_ID)
                and (area_ID == str(pccl.PC_IH_NCD_MST_ID)) 
                and (costing_sect_ID == pcc.PC_PPLI_TOBACCO_CTRL_COST_SECT_ID)
                and (costing_subsect_ID == pcc.PC_RAISE_TAXES_PPLI_COSTING_SUBSECT_ID)
            ):
                for t in acu.GBRange(1, num_years):
                    pcmvu.set_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_PHASE_ROW_PPLI_CURR_ID, t - 1, phase_ID)
                # Make the final year full implementation for all countries.
                pcmvu.set_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_PHASE_ROW_PPLI_CURR_ID, num_years - 1, 
                    pcc.PC_FULL_IMP_PHASE_100_PPLI_MST_ID)
                
                pcmvu.set_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_INTENSITY_ROW_PPLI_CURR_ID, num_years - 1, 
                    pcu.get_PPLI_intensity_value(pcc.PC_FULL_IMP_PHASE_100_PPLI_MST_ID))
            
            else:
                for t in acu.GBRange(1, num_years):
                    pcmvu.set_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_PHASE_ROW_PPLI_CURR_ID, t - 1, phase_ID)
                    pcmvu.set_PC_costing_subsect_PPLI_phase_intensity(costing_subsect_rec, pcc.PC_INTENSITY_ROW_PPLI_CURR_ID, t - 1, 
                        pcu.get_PPLI_intensity_value(phase_ID))

#######################################################################################################
#                                                                                                     #
#                                       PPLI taxation                                                 #
#                                                                                                     #
#######################################################################################################

def PC_set_price_elasticity_params(PPLI_taxation_DB, tobacco_costing_sect_rec):
    if tobacco_costing_sect_rec != gbc.GB_NOT_FOUND:
        pcmvu.set_PC_costing_sect_price_elasticity_PPLI(tobacco_costing_sect_rec, pcc.PC_TAX_PARAMS_PPLI_CURR_ID,
            PPLI_taxation_DB[pcdbk.PC_PRICE_ELASTICITY_PARAMS_KEY_PPLITDB])





from setuptools import find_packages, setup


subpackages = [pkg for pkg in find_packages(where=".") if not pkg.startswith("SpectrumCommon")]

packages = ["SpectrumCommon"]
package_dir = {"SpectrumCommon": "."}

for package_name in subpackages:
    full_name = f"SpectrumCommon.{package_name}"
    packages.append(full_name)
    package_dir[full_name] = package_name.replace(".", "/")


setup(
    packages=packages,
    package_dir=package_dir,
)




import numpy as np
from os import environ

from AvenirCommon.Database import GB_get_db_json, GB_file_exists
from AvenirCommon.Util import formatCountryFName, GBGetKeyValue

from SpectrumCommon.Const.GB import *
from SpectrumCommon.Const.DP import *
from SpectrumCommon.Const.TB import *
from SpectrumCommon.Const.GB.GBVersions import (TB_WHO_DB_VERSION,
                                               TB_DIAGNOSTICS_VERSION,
                                               TB_DYNAMICAL_MODEL_VERSION,
                                               TB_FORT_INPUT_VERSION,
                                               TB_FORT_OUTPUT_VERSION)
from SpectrumCommon.ServerUtils.ExceptionUtils import SpectrumException
from SpectrumCommon.Modvars.TB.TBModvDefs import *
from SpectrumCommon.Modvars.TB.TBNotifDefs import *
from SpectrumCommon.Modvars.TB.TBHouseholdDefs import *
from SpectrumCommon.Modvars.TB.TBHighRiskGrpDefs import *
from SpectrumCommon.Modvars.TB.TBResistanceDefs import *
from SpectrumCommon.Modvars.TB.TBPreventionDefs import *
from SpectrumCommon.Modvars.TB.TBARTDefs import *
from SpectrumCommon.Modvars.TB.TBScreeningDefs import *


#@autoreload
def init_modvars(countryID, first_year, final_year,  extra):
    
    # year options
   
    # base_year = TB_Base_Year if first_year!=2021 else 2021
    if (first_year<TB_Min_First_Year):
        raise SpectrumException(GB_SW_BADPROJPARAM, message=f'TB first year is less than {str(TB_Min_First_Year)}')
    
    if ( first_year>TB_Max_First_Year):
        raise SpectrumException(GB_SW_BADPROJPARAM, message=f'TB first year is greater than {str(TB_Max_First_Year)}')

    base_year = first_year
    # who_db_vsn = '_V6' # if first_year!=2021 else '_V1'
    # fort_vsn = '_V7' #if first_year!=2021 else '_V2'
    dp_t = base_year-GB_BaseYear
    subnatCode = GBGetKeyValue(extra, 'subNatCode', 0)
    connection = environ['AVENIR_SW_DEFAULT_DATA_CONNECTION']

    year_len = final_year-base_year + 1

    fort_slice = slice(base_year-TB_FirstHist_Year, min(TB_Fort_Final_Year, final_year) -TB_FirstHist_Year+1)
    tb_slice  = slice(0, min(TB_Fort_Final_Year, final_year)-base_year+1)

    ###############################################################################################
    ############################## Database defaults ##############################################
    ###############################################################################################
    
    db =  GB_get_db_json(connection, 
                         'tuberculosis', 
                         f'countries\\{countryID}_{TB_WHO_DB_VERSION}.JSON')
    diagnostic_db = GB_get_db_json(connection, 
                                   'tuberculosis', 
                                   f'diagnosticDefaults_{TB_DIAGNOSTICS_VERSION}.JSON')
    # screening_db  = GB_get_db_json(connection, 'tuberculosis', 'screeningDefaults_V1.JSON')

    
    first_idx = base_year-db['projections']['startYear']
    final_idx = final_year-db['projections']['startYear']
    burden_first_idx = base_year-2000
    burden_final_idx = final_year-2000

        # TB_HisIncidenceTag  : hist_inci,
        # TB_HisMortalityTag  : ,
    # flatline defaults 
    TB_flatlineDB(db, 'notification', final_idx)
    TB_flatlineDB(db, 'prevalencePct', final_idx)
    TB_flatlineDB(db, 'prevalence', final_idx)
    TB_flatlineDB(db, 'pop', final_idx)
    TB_flatlinBurden(db, 'e_inc_num', final_idx)
    TB_flatlinBurden(db, 'e_mort_num', final_idx)
    
    #subnat notification
    pop, pop_15_plus = TB_getPopDefaults(connection, countryID, subnatCode, base_year, final_year)

    ###############################################################################################
    ###############################################################################################
    ###############################################################################################


    hh_prev_methods, hh_prev_method_IDs = HouseholdPreventionMethodsUsed()
    ART_prev_methods, ART_prev_method_IDs  = ARTPreventionMethodsUsed()
    HRG_prev_methods, HRG_prev_method_IDs  = HRGPreventionMethodsUsed()

    fort_inputs = fort.get_tb_fort_inputs(countryID, final_year, tb_slice, fort_slice, None, None, None)
    notif_base_pct, notif_scale, c_ep, bac_pos_pct = create_notification_defaults_2022(countryID, db['projections']['notification'], db['Noti_Distribution'], TB_FinalHist_Year-db['projections']['startYear'])
    notif, notif_split, notif_pct, fort_outputs = create_notification_defaults(countryID, db['projections']['notification'], db['Noti_Distribution'], first_year, first_idx, final_idx, year_len, 
                                                                               notif_base_pct, tb_slice, fort_slice, TB_FORT_OUTPUT_VERSION, notif_scale, 1)

    rr_profile, rp_Rif = TB_RRProfileDefaults(db, notif_pct, year_len)

    ## Screening Algorithm secions
    screen_map = TB_screening_sens_spec_map()

    patient_screen_selected = TB_screening_alg_patient_defaults()
    patient_sensitivity = [screen_map[screen_id][TB_Sensitivity_index] for screen_id in patient_screen_selected]
    patient_specificity = [screen_map[screen_id][TB_Specificity_index] for screen_id in patient_screen_selected]

    household_screen_selected = TB_screening_alg_household_defaults()
    household_sensitivity = [screen_map[screen_id][TB_Sensitivity_index] for screen_id in household_screen_selected]
    household_specificity = [screen_map[screen_id][TB_Specificity_index] for screen_id in household_screen_selected]

    art_screen_selected = TB_screening_alg_art_defaults()

    art_not_serious_sensitivity = [screen_map[screen_id][TB_Sensitivity_index] for screen_id in art_screen_selected[TB_ARTNotServere]]
    art_not_serious_specificity = [screen_map[screen_id][TB_Specificity_index] for screen_id in art_screen_selected[TB_ARTNotServere]]
    art_serious_sensitivity = [screen_map[screen_id][TB_Sensitivity_index] for screen_id in art_screen_selected[TB_ARTSevereIllness]]
    art_serious_specificity = [screen_map[screen_id][TB_Specificity_index] for screen_id in art_screen_selected[TB_ARTSevereIllness]]
    art_sensitivity = [ art_not_serious_sensitivity, art_serious_sensitivity]
    art_specificity = [ art_not_serious_specificity, art_serious_specificity]

    hrg_screen_selected = TB_screening_alg_hrg_defaults()
    hrg_sensitivity = [screen_map[screen_id][TB_Sensitivity_index] for screen_id in hrg_screen_selected]
    hrg_specificity = [screen_map[screen_id][TB_Specificity_index] for screen_id in hrg_screen_selected]

    incidence = np.zeros(year_len)
    incidence[tb_slice] = fort_outputs['iMid'][fort_slice]
    # flatline no longer needed, taking all of forts values
    # incidence[TB_Fort_Final_Year-base_year:] = incidence[TB_Fort_Final_Year-base_year] # flatline incidence from 2030 onwards
    
    mortality = np.zeros(year_len)
    mortality[tb_slice] = fort_outputs['mMid'][fort_slice]

    # prevalence = calc_prevalence_from_notif_inci(notif, incidence)
    prevalence = np.zeros(year_len)
    prevalence[tb_slice] = fort_outputs['pMid'][fort_slice]
    prev_pct = prevalence/db['projections']['pop'][first_idx:final_idx+1]

    if GB_file_exists(connection, 'tuberculosis', f'dynamicalmodel\\countries\\{countryID}_{TB_DYNAMICAL_MODEL_VERSION}.JSON'):
        dyn_db =  GB_get_db_json(connection, 'tuberculosis', f'dynamicalmodel\\countries\\{countryID}_{TB_DYNAMICAL_MODEL_VERSION}.JSON')
    else:
        dyn_db =  GB_get_db_json(connection, 'tuberculosis', f'dynamicalmodel\\countries\\NGA_{TB_DYNAMICAL_MODEL_VERSION}.JSON')
    # p = dyn_db['p']
    # r = dyn_db['r']
    # dyn_db['prm']['r'] = r
    # dyn_db['prm']['p'] = p
    prm = dyn_db['prm']
    # p = {		
    #     'rfbeta_mdr'  : 0,
    #     'rfbeta_hiv'  : 0,
    #     'imm'         : np.zeros((TB_DynHIVARTLen)).tolist(),
    #     'HIV_relrate' : 0, 
    #     'pu'          : 0,
    #     'Dx'          : np.zeros((TB_DynSectorLen)).tolist(),
    #     'Tx_init'     : np.zeros((TB_DynSectorLen)).tolist(),
    #     'TX_complete' : np.zeros((TB_DynSectorLen)).tolist(),
    #     'cure'        : np.zeros((TB_DynSectorLen)).tolist(),
    #     'MDR_rec2015' : np.zeros((TB_DynSectorLen)).tolist(),
    #     'MDR_rec2022' : np.zeros((TB_DynSectorLen)).tolist(),
    #     'Tx_init2'    : np.zeros((TB_DynSectorLen)).tolist(),
    #     'SL_trans'    : np.zeros((TB_DynSectorLen)).tolist(), 
    #     'tsrsl'       : np.zeros((TB_DynSectorLen)).tolist(),
    #     'cure2'       : np.zeros((TB_DynSectorLen)).tolist(),
    #     'PT_PLHIV'    : np.zeros((TB_DynHIVARTLen)).tolist(),
    #     'VE'          : np.zeros((TB_DynExposureLen)).tolist(),    
    # }
    # r = {
    #     'beta'         : 0,
    #     'reactivation' :  np.zeros((TB_DynAgeLen, TB_DynHIVARTLen)).tolist(),
    #     'progression'  :  np.zeros((TB_DynAgeLen, TB_DynHIVARTLen)).tolist(),
    #     'LTBI_stabil'  : np.zeros((TB_DynHIVARTLen)).tolist(),
    #     'relapse'      : np.zeros((TB_DynRelapseLen)).tolist(),
    #     'self_cure'    : np.zeros((TB_DynHIVARTLen)).tolist(),
    #     'mort_TB'      : np.zeros((TB_DynHIVLen)).tolist(),
    #     'HIV_mort'     : 0,
    #     'sym'          : 0,
    #     'cs'           :  np.zeros((TB_DynAgeLen)).tolist(),
    #     'cs2'          :  np.zeros((TB_DynAgeLen)).tolist(),
    #     'cs3'          :  np.zeros((TB_DynAgeLen)).tolist(),
    #     'Dx'           : 0,
    #     'Tx'           : 0,
    #     'Tx2'          : 0,
    #     'default'      : np.zeros((TB_DynSectorLen)).tolist(),
    #     'MDR_acqu'     : 0,
    #     'default2'     : np.zeros((TB_DynSectorLen)).tolist(),
    #     # HIV related
    #     'ART_init'     : 0,
    #     # Demographics
    #     'mort'         : np.zeros((TB_DynAgeLen)).tolist(),
    #     'ageing'        : np.zeros((TB_DynAgeLen)).tolist(),
    #     'vacc'         : 0,
    #     'waning'       : 0, 
    # }
    used_HIV_incd=0
    # prm = {
    #     'cm': np.zeros((TB_DynAgeLen,TB_DynAgeLen)).tolist(),
    #     'rHIV': 0,
    #     'ART_start':0,
    #     'r': r, 
    #     'p': p,
    #     'usedHIVIncd' : used_HIV_incd
    # }
    # prm = TB_PRM_Default_Nigeria
    prm['usedHIVIncd' ] = used_HIV_incd
    
    r_include = {
                'beta'         : True,
                'reactivation' : False,
                'progression'  : False,
                'LTBI_stabil'  : False,
                'relapse'      : False,
                'self_cure'    : True,
                'mort_TB'      : True,
                'HIV_mort'     : True,
                'sym'          : True,
                'cs'           : True,
                'cs2'          : True,
                'cs3'          : True,
                'Dx'           : False,
                'Tx'           : False,
                'Tx2'          : False,
                'default'      : False,
                'MDR_acqu'     : True,
                'default2'     : False,
                # HIV related
                'ART_init'     : True,
                # Demographics
                'mort'         : False,
                'ageing'       : False,
                'vacc'         : False,
                'waning'       : False, 
    }
    p_include = {		
        'rfbeta_mdr'  : True,
        'rfbeta_hiv'  : True,
        'imm'         : False,
        'HIV_relrate' : True,
        'pu'          : True,
        'Dx'          : True,
        'Tx_init'     : False,
        'TX_complete' : True,
        'cure'        : False,
        'MDR_rec2015'  : False,
        'MDR_rec2022'  : True,
        'Tx_init2'    : False,
        'SL_trans'    : False,
        'tsrsl'       : False,
        'cure2'       : False,
        'PT_PLHIV'    : False,
        'VE'          : False,
    }
    used_HIV_incd_include = False
    prm_include = {
                'cm': False,
                'rHIV': False,
                'ART_start':False,
                'r' : r_include,
                'p' : p_include,
                'UsedHIVIncd' : used_HIV_incd_include
    }
    
    r_range = {
        'beta'         : [0, 40],
        'reactivation' : [],
        'progression'  : [],
        'LTBI_stabil'  : [],
        'relapse'      : [],
        'self_cure'    : [0.85/6, 1.15/6], # 1/6*[0.85 - 1.15]
        'mort_TB'      : [[0, 2/6], [0, 100/6]],  
        'HIV_mort'     : [0, 10],
        'sym'          : [0.1, 100.0],
        'cs'           : [0.1, 100.0],
        'cs2'          : [1, 24],
        'cs3'          : [1, 24],
        'Dx'           : [],
        'Tx'           : [],
        'Tx2'          : [],
        'default'      : [],
        'MDR_acqu'     : [0.0, 0.06],
        'default2'     : [],
        # HIV related
        'ART_init'     : [0, 10],
        # Demographics
        'mort'         : [],
        'ageing'       : [],
        'vacc'         : [],
        'waning'       : [],
    }
    used_HIV_incd_range = []
    p_range = {		
        'rfbeta_mdr'  : [0.0, 1.0],
        'rfbeta_hiv'  : [0.0, 1.0],
        'imm'         : [],
        'HIV_relrate' : [1, 100],
        'pu'          : [],
        'Dx'          : [[0.75, 0.9], [0.1, 0.3]],
        'Tx_init'     : [],
        'TX_complete' : [[0.75, 0.95], [0.4, 0.8]],
        'cure'        : [],
        'MDR_rec2015' : [],
        'MDR_rec2022' : [[0, 1], [0, 1]],
        'Tx_init2'    : [],
        'SL_trans'    : [],
        'tsrsl'       : [],
        'cure2'       : [],
        'PT_PLHIV'    : [],
        'VE'          : [],
    }
    prm_range = {
        'cm'        : [],
        'rHIV'      : [],
        'ART_start' : [],
        'p':p_range,
        'r':r_range,
        'usedHIVIncd':used_HIV_incd_range
    }
    #modvar creation list
    return {
        TB_BaseYearTag: base_year,
        TB_DataSourceTag: extra['countryDetails']['sources']['TBSource'],
        TB_UseOldFailsafeTag: False, #only true for old 2021 projections 
        TB_UseOldIPTag: False, #only true for old 2022 projections 

        TB_HistNotifStartYearTag   : db['notifications']['startYear'],
        TB_HistNotifTag            : {key:db['notifications'][key] for key in TB_NOTIF_DB_KEYS},
        TB_HistNotiDistributionTag : {key:db['Noti_Distribution'][key] for key in TB_NOTIF_DIST_KEYS},
        
        TB_ScreeningAlgMapTag              : screen_map,

        # From the wireframes
        TB_PulmonaryAgeTag                 : TB_Pulmonary_list,
        TB_PrevalenceTag                   : np.full((TB_Pulmonary_count, year_len), 0.10),
        TB_PatientScreenSensitivityTag     : patient_sensitivity, # np.full((TB_Pulmonary_count), 0.90),
        TB_PatientScreenSpecificityTag     : patient_specificity, # np.full((TB_Pulmonary_count), 0.99),
        TB_PatientScreenAlgOptionsTag      : TB_screening_alg_patient_options(),
        TB_PatientScreenAlgSelectedTag     : patient_screen_selected,

        TB_NumClinicallyAssessedTag        : np.zeros((TB_Pulmonary_count, year_len)),
        TB_NumReferredTag                  : np.zeros((TB_Pulmonary_count, year_len)),
        TB_NumScreenedToReferTag           : np.zeros((TB_Pulmonary_count, year_len)),
        TB_PercTrueCasesTag                : np.zeros((TB_Pulmonary_count, year_len)),

        TB_PatientDiagCaseValuesTag            : diagnostic_db['patients']['values'],
        TB_PatientDiagCaseIDsTag               : diagnostic_db['patients']['IDs'],
        TB_PatientDiagCaseMethodsTag           : diagnostic_db['patients']['methods'],
        TB_PatientDiagCaseSensitivityTag       : diagnostic_db['patients']['sensitivity'],
        TB_PatientDiagCaseSpecificityTag       : diagnostic_db['patients']['specificity'],

        TB_PatientDiagSensitivityTag       : np.full((TB_Pulmonary_count), 0.84), # db['Sensitivity']['Patient'], # 
        TB_PatientDiagSpecificityTag       : np.full((TB_Pulmonary_count), 0.97), # db['Specificity']['Patient'], # 
        TB_NumDiagnosedPTBTag              : np.zeros((TB_Pulmonary_count, year_len)),
        TB_NumConfirmedPTBTag              : np.zeros((TB_Pulmonary_count, year_len)),
        TB_NumDiagnosedTBTag               : np.zeros((TB_Pulmonary_count, year_len)),
        TB_NumInitTreatmentTag             : np.zeros((TB_Pulmonary_count, year_len)),

        TB_PatientScreeningOutputTag       : np.zeros((TB_Pulmonary_count, TB_ScreenOutputLen, year_len)),
        TB_PatientDiagnosticOutputTag      : np.zeros((TB_Pulmonary_count, TB_DiagnosisLen, year_len)),
        TB_PatientTargetPopTag             : np.zeros((TB_Pulmonary_count, TB_PatientTargPopLen, year_len)),
                    
        TB_PatientCustStepMenu : TB_Cust_patient_menu,
        TB_PatientCustStep1 : TB_Cust_patient_default,
        TB_PatientCustStep2 : TB_Cust_patient_default,
        TB_PatientCustParallel : np.full((TB_Pulmonary_count), True).tolist(),
        TB_ScreenCustomTag : {}, #empty dict of custom ids
        TB_DiagnosticCustomTag : {},
        

        # provider households
        
        # Household 
        ## Household demographics
        ### Demographics
        TB_HHSizeTag: db['estimates'].get('e_hh_size', [0])[0],# Average household size (#)
        TB_HHPropU5Tag: 0.15,#  Proportion of household that is u5 (# )
        TB_HHProp5_14Tag: 0.10,# Proportion of household that is 5-14 yrs 
        TB_HHCasesinIndexTag: 1.2,# Average number of TB cases in household with at least one case (#)

        ### TB infection characteristics
        TB_HHPropU5LTBITag: np.full((year_len), 0.35), # Proportion of u5s with TB infection in household with TB infection (# )
        TB_HHPropO5LTBITag: np.full((year_len), 0.45), # Proportion of other household members with TB infection in household with active case (# )

        #### TB disease characteristics
        TB_HHPropU5TBTag:  np.full((year_len), 0.05),  # Proportion of�u5s with TB disease in household with TB disease (# )
        TB_HHPropU5PTBTag:  np.full((year_len), 1-c_ep),  # , where t is final year of WHO data
        TB_HHPropO5TBTag:  np.full((year_len), 0.05),  # Proportion of other household members with TB disease in household with TB disease (# )
        TB_HHPropO5PTBTag: np.full((year_len), 1-c_ep), # Proportion with pulmonary TB, PTB (# )

        ##     Index cases
        TB_HHNumNewBactPosTag : notif*bac_pos_pct, # Number of new bacteriologically positive TB cases (adult), total notification from database 
        TB_HHNumNewBactNegTag : notif*(1-bac_pos_pct),# Number of new bacteriologically negative TB cases (adult),  total notification from database x 1- self.Noti_Data["new_labconf" + "ret_rel_labconf" ][TB_noti_perc][t]

        ## household contact tracing coverage
        TB_HHPIndexCasesScreenedTag: np.full((year_len), 1.00), # Proportion of households of index cases screened/evaluated
        TB_HHPropBACnScreenedTag: np.full((year_len), 0.20),  # Proportion of contacts of new bacteriologically negative cases screened/evaluated
        
        ## Tab Household contacts by age
        TB_HHContacts                : np.zeros((TB_HHAgeLen, TB_ContactLen, year_len)),   
        
        ## Tab Household screening
        ### Household proportion
        TB_HHProportionTag           : np.full((TB_HHAgeLen, year_len), 0.5), 

        ### Household sensitivity amd specificity
        TB_HHScreenSensitivityTag : household_sensitivity, # np.full((TB_HHAgeLen), 0.90),
        TB_HHScreenSpecificityTag : household_specificity, # np.full((TB_HHAgeLen), 0.99),  
        TB_HHScreenAlgOptionsTag  : TB_screening_alg_household_options(),
        TB_HHScreenAlgSelectedTag : household_screen_selected,


        ### Household screening outputs
        TB_HHNumScreenedTag           : np.zeros((TB_HHAgeLen, year_len)),
        TB_HHNumReferredTag           : np.zeros((TB_HHAgeLen, year_len)),
        TB_HHNumScreenedToReferTag    : np.zeros((TB_HHAgeLen, year_len)),
        TB_HHPercTrueCasesTag         : np.zeros((TB_HHAgeLen, year_len)),     

        ## Tab Household diagnostic evaluation
        ### TB Case type split by treatment and sputum status
        TB_HHDiagCaseValuesTag               : diagnostic_db['households']['values'],
        TB_HHDiagCaseIDsTag                  : diagnostic_db['households']['IDs'],
        TB_HHDiagCaseMethodsTag              : diagnostic_db['households']['methods'],
        TB_HHDiagCaseSensitivityTag          : diagnostic_db['households']['sensitivity'],
        TB_HHDiagCaseSpecificityTag          : diagnostic_db['households']['specificity'],

        ### Household sensitivity and specificity

        TB_HHDiagSensitivityTag : np.full((TB_HHAgeLen),0.84), # db['Sensitivity']['HouseHold'], # np.full((TB_HHAgeLen), 0.90),
        TB_HHDiagSpecificityTag : np.full((TB_HHAgeLen),0.97), # db['Specificity']['HouseHold'], # np.full((TB_HHAgeLen), 0.99),  

        ### Diagnosis outputs
        TB_HHNumDiagnosedPTBTag       : np.zeros((TB_HHAgeLen, year_len)),             
        TB_HHNumConfirmedPTBTag       : np.zeros((TB_HHAgeLen, year_len)),  

        ## Tab Household disease and treatment
        TB_HHPropLinkedTreatTag       : np.full((TB_HHAgeLen, year_len), 1.00),           
        TB_HHNumInitTreatmentTag      : np.zeros((TB_HHAgeLen, year_len)),   
        
        ## Tab Household prevention
        TB_HHPrvntInActiveEligibleTag  : np.zeros((TB_HHAgeLen, year_len)),     
        TB_HHPrvntReceivePTPrevTag    : np.full((TB_HHAgeLen, year_len), 0.00),   
        TB_HHPropPrvntPresumpTag       : TB_HHPropPrvntPresumpDefaults(year_len), # np.full((TB_HHAgeLen, year_len), 0.00),  # 100 only fot children   
        TB_HHPropPrvntTestedTag        : TB_HHPropPrvntTested(year_len),  # np.full((TB_HHAgeLen, year_len), 0),    
        
        TB_HHPrvntMethodsTag           : hh_prev_methods,    
        TB_HHPrvntMethodIDsTag         : hh_prev_method_IDs, 
        TB_HHPropEligiblePrvntLinkTag  : np.full((TB_HHAgeLen, year_len), 1.00),  
        
        TB_HHNumberOnTPTTag         : np.zeros((TB_HHAgeLen, year_len)),  
        TB_HHTPTCompleteRateTag     : np.full((TB_HHAgeLen, year_len), 1.0),
        TB_HHNumberCompleteTPTTag   : np.zeros((TB_HHAgeLen, year_len)),  

        ## Tab Household output
        TB_HHScreeningOutputTag  : np.zeros((TB_HHAgeLen, TB_ScreenOutputLen, year_len)),   
        TB_HHDiagnosticOutputTag : np.zeros((TB_HHAgeLen, TB_DiagnosisLen, year_len)),   
        TB_HHTargetPopsTag       : np.zeros((TB_HHAgeLen, TB_TargetPopLen, year_len)),   
   

        TB_HHCustStepMenu : TB_Cust_HH_menu,
        TB_HHCustStep1 : TB_Cust_HH_default,
        TB_HHCustStep2 : TB_Cust_HH_default,
        TB_HHCustParallel : np.full((TB_HHAgeLen), True).tolist(),
        
        ## Tab ART cohort sizes and TB characteristics
        ### ART cohort sizes
        TB_ARTCohortSizesTag         : TB_ARTCohortsDefaults(pop, base_year, final_year),

        ### TB characteristics
        TB_ARTCharacteristicsTag     : TB_ARTCharacteristicDefaults(c_ep, year_len),
        
        ## Tab ART screening
        ### ART proportion
        TB_ARTProportionTag           : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 0.50), 

        ### ART sensitivity amd specificity
        TB_ARTScreenSensitivityTag : art_sensitivity, # np.full((TB_ARTAgeLen), 0.90),
        TB_ARTScreenSpecificityTag : art_specificity, # np.full((TB_ARTAgeLen), 0.99),  
        TB_ARTScreenAlgOptionsTag  : TB_screening_alg_art_options(),
        TB_ARTScreenAlgSelectedTag : art_screen_selected,

        ### ART screening outputs
        TB_ARTNumScreenedTag           : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),
        TB_ARTNumReferredTag           : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),
        TB_ARTNumScreenedToReferTag    : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),
        TB_ARTPercTrueCasesTag         : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),     

        ## Tab ART diagnostic evaluation
        ### TB Case type split by treatment and sputum status
        TB_ARTDiagCaseValuesTag               : [diagnostic_db['art']['values']]*TB_ARTSevereLen,
        TB_ARTDiagCaseIDsTag                  : [diagnostic_db['art']['IDs']]*TB_ARTSevereLen,
        TB_ARTDiagCaseMethodsTag              : [diagnostic_db['art']['methods']]*TB_ARTSevereLen,
        TB_ARTDiagCaseSensitivityTag          : [diagnostic_db['art']['sensitivity']]*TB_ARTSevereLen,
        TB_ARTDiagCaseSpecificityTag          : [diagnostic_db['art']['specificity']]*TB_ARTSevereLen,

        ### ART sensitivity amd specificity
        TB_ARTDiagSensitivityTag : np.full((TB_ARTSevereLen, TB_ARTAgeLen),0.84), # np.full((TB_ARTAgeLen), 0.90),
        TB_ARTDiagSpecificityTag : np.full((TB_ARTSevereLen, TB_ARTAgeLen),0.97), # np.full((TB_ARTAgeLen), 0.99), 

        ### Diagnosis outputs
        TB_ARTNumDiagnosedPTBTag       : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),             
        TB_ARTNumConfirmedPTBTag       : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),  

        ## Tab ART disease and treatment
        TB_ARTPropLinkedTreatTag       : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 1.00),           
        TB_ARTNumInitTreatmentTag      : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),   
        
        ## Tab ART prevention
        TB_ARTPrvntInActiveEligibleTag  : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),   
        TB_ARTPrvntReceivePTPrevTag     : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 0.00),     
        TB_ARTPropPrvntPresumpTag       : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 0.50),     
        TB_ARTPropPrvntTestedTag        : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 0),    
        
        TB_ARTPrvntMethodsTag           : ART_prev_methods,    
        TB_ARTPrvntMethodIDsTag         : ART_prev_method_IDs, 
        TB_ARTPropEligiblePrvntLinkTag  : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 1.00),  
        
        TB_ARTNumberOnTPTTag         : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),  
        TB_ARTTPTCompleteRateTag     : np.full((TB_ARTSevereLen, TB_ARTAgeLen, year_len), 1.0),
        TB_ARTNumberCompleteTPTTag   : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, year_len)),  
        
        ## Tab ART output
        TB_ARTScreeningOutputTag  : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, TB_ScreenOutputLen, year_len)),   
        TB_ARTDiagnosticOutputTag : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, TB_DiagnosisLen, year_len)),   
        TB_ARTTargetPopsTag             : np.zeros((TB_ARTSevereLen, TB_ARTAgeLen, TB_TargetPopLen, year_len)), 
        
        TB_ARTCustStepMenu : TB_Cust_ART_menu,
        TB_ARTCustStep1 : TB_Cust_ART_default,
        TB_ARTCustStep2 : TB_Cust_ART_default,
        TB_ARTCustParallel : np.full((TB_ARTSevereLen, TB_ARTAgeLen), True).tolist(),  

        ## Tab HRG demographics
        TB_HRGDemographicsTag      : TB_HighRiskGroupDemographicDefaults(pop_15_plus, c_ep, db['projections']['prevalencePct'][first_idx:final_idx+1], year_len), 
        ## Tab HRG screening
        ### HRG proportion
        TB_HRGProportionTag        : np.full((TB_HRG_Len, year_len), 0.00), 

        ### HRG sensitivity amd specificity
        TB_HRGScreenSensitivityTag : hrg_sensitivity,
        TB_HRGScreenSpecificityTag : hrg_specificity, 
        TB_HRGScreenAlgOptionsTag  : TB_screening_alg_hrg_options(),
        TB_HRGScreenAlgSelectedTag : hrg_screen_selected, 

        ### HRG screening outputs
        TB_HRGNumScreenedTag           : np.full((TB_HRG_Len, year_len), 12.3),
        TB_HRGNumReferredTag           : np.full((TB_HRG_Len, year_len), 1.2),
        TB_HRGNumScreenedToReferTag    : np.full((TB_HRG_Len, year_len), 11.1),
        TB_HRGPercTrueCasesTag         : np.full((TB_HRG_Len, year_len), 0.90),     

        ## Tab HRG diagnostic evaluation
        ### TB Case type split by treatment and sputum status
        TB_HRGDiagCaseValuesTag               : [diagnostic_db['highriskgroups']['values'][0]]*TB_HRG_Len,
        TB_HRGDiagCaseIDsTag                  : [diagnostic_db['highriskgroups']['IDs'][0]]*TB_HRG_Len,
        TB_HRGDiagCaseMethodsTag              : [diagnostic_db['highriskgroups']['methods'][0]]*TB_HRG_Len,
        TB_HRGDiagCaseSensitivityTag          : [diagnostic_db['highriskgroups']['sensitivity'][0]]*TB_HRG_Len,
        TB_HRGDiagCaseSpecificityTag          : [diagnostic_db['highriskgroups']['specificity'][0]]*TB_HRG_Len,

        ### HRG sensitivity amd specificity
        TB_HRGDiagSensitivityTag : np.full((TB_HRG_Len),0.84), # np.full((TB_HRG_Len),  db['Sensitivity']['HRG'][0]),
        TB_HRGDiagSpecificityTag : np.full((TB_HRG_Len),0.97), # np.full((TB_HRG_Len),  db['Specificity']['HRG'][0]),  

        ### Diagnosis outputs
        TB_HRGNumDiagnosedPTBTag       : np.full((TB_HRG_Len, year_len), 1.3),             
        TB_HRGNumConfirmedPTBTag       : np.full((TB_HRG_Len, year_len), 39.3),  

        ## Tab HRG disease and treatment
        TB_HRGPropLinkedTreatTag       : np.full((TB_HRG_Len, year_len), 1.00),           
        TB_HRGNumInitTreatmentTag      : np.full((TB_HRG_Len, year_len), 1.3),   
        
        ## Tab HRG prevention
        TB_HRGPrvntInActiveEligibleTag  : np.full((TB_HRG_Len, year_len), 0.497),   
        TB_HRGPrvntReceivePTPrevTag     : np.full((TB_HRG_Len, year_len), 0.00),   
        TB_HRGPropPrvntPresumpTag       : np.full((TB_HRG_Len, year_len), 0.00),     
        TB_HRGPropPrvntTestedTag        : np.full((TB_HRG_Len, year_len), 1),    
        
        TB_HRGPrvntMethodsTag           : HRG_prev_methods,    
        TB_HRGPrvntMethodIDsTag         : HRG_prev_method_IDs, 
        TB_HRGPropEligiblePrvntLinkTag  : np.full((TB_HRG_Len, year_len), 1.00),  
        
        TB_HRGNumberOnTPTTag         : np.zeros((TB_HRG_Len, year_len)),  
        TB_HRGTPTCompleteRateTag     : np.full((TB_HRG_Len,  year_len), 1.0),
        TB_HRGNumberCompleteTPTTag   : np.zeros((TB_HRG_Len, year_len)), 
        
        ## Tab HRG output
        TB_HRGScreeningOutputTag       : np.zeros((TB_HRG_Len, TB_ScreenOutputLen, year_len)),   
        TB_HRGDiagnosticOutputTag      : np.zeros((TB_HRG_Len, TB_DiagnosisLen, year_len)), 
        TB_HRGTargetPopsTag            : np.zeros((TB_HRG_Len, TB_TargetPopLen, year_len)), 

        TB_HRGCustStepMenu : TB_Cust_HRG_menu,
        TB_HRGCustStep1 : TB_Cust_HRG_default,
        TB_HRGCustStep2 : TB_Cust_HRG_default,
        TB_HRGCustParallel : np.full((TB_HRG_Len), True).tolist(), 


        TB_ResistProfileTag  : rr_profile,
        TB_ResistPropNonSevereTag  : np.full((TB_NewRetreatLen, TB_ResistAgeLen, year_len), 0.95),
        TB_ResistPropLungSurgeryTag  : np.full((TB_NewRetreatLen, TB_ResistAgeLen, year_len), 0.05),
        TB_ResistMeningitisTag  : np.full((TB_NewRetreatLen, TB_ResistAgeLen, year_len), 0.05),
        TB_ResistPropTreatExhaustTag  : np.full((TB_NewRetreatLen, TB_ResistAgeLen, year_len), 0.05),

        TB_PatientTestNumPTB  : np.full((year_len), 95),
        TB_PatientTestNumXDR  : np.full((year_len), 5),
        TB_PatientTestNumEPTB : np.full((year_len), 10), 

        TB_CoverageTag : TB_ResistanceCoverageDefaults(year_len),
        TB_ResistNumToBeTestedTag : np.zeros((TB_NumTestLen, year_len)), 
        TB_ResistNumTestedTag : np.zeros((TB_NumTestLen, year_len)), 
        TB_ResistNumConfirmedTag : np.full((TB_NumConfResistLen, TB_ResistAgeLen, year_len), 50), 
        TB_ResistNumNotConfirmedTag : np.full((TB_NumConfResistLen, TB_ResistAgeLen, year_len), 0), 


        ## Prevention of TB
        TB_PropPreventionTag : TB_PreventionDefaults(rp_Rif ,year_len),

        ## Vaccination
        TB_BCGImmunizationTag : np.full((year_len), 1.00),
        TB_InfantVaccineTag : np.full((year_len), 1.00),
        TB_AdolAdultVaccineTag : np.full((year_len), 1.00),

        ## Palliative care
        TB_PalliativeCareTag : np.zeros((TB_ResistAgeLen, year_len)),

        TB_BCGTargetPopTag          : np.full((year_len), 0),
        TB_InfantVaccTargPopTag     : np.full((year_len), 0),
        TB_AdultVaccTargPopTag      : np.full((year_len), 0),
        TB_PalliativeCareTargPopTag : np.full((TB_ResistAgeLen, year_len), 0),




        #Extra target populations
        TB_PreventionTargetTag : np.zeros((TB_ResistanceLen, TB_ResistAgeLen, year_len)),
        
        TB_RR_DST_PTBTag     : np.zeros(year_len),
        TB_RR_DST_ETBTag     : np.zeros(year_len),
        TB_INH_DST_PTBTag    : np.zeros(year_len),
        TB_INH_DST_ETBTag    : np.zeros(year_len),
        TB_BDQ_DST_PTBTag    : np.zeros(year_len),
        TB_BDQ_DST_ETBTag    : np.zeros(year_len),
        TB_FQ_DST_PTBTag     : np.zeros(year_len),
        TB_FQ_DST_ETBTag     : np.zeros(year_len),
        TB_BDQ_FQ_DST_PTBTag : np.zeros(year_len),
        TB_BDQ_FQ_DST_ETBTag : np.zeros(year_len),
        TB_LZD_RR_DST_PTBTag : np.zeros(year_len),
        TB_LZD_RR_DST_ETBTag : np.zeros(year_len),
        TB_LZD_FQ_DST_PTBTag : np.zeros(year_len),
        TB_LZD_FQ_DST_ETBTag : np.zeros(year_len),
        TB_PZA_DSTTag        : np.zeros(year_len),
        TB_ETO_DSTTag        : np.zeros(year_len),
        TB_EMC_DSTTag        : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_U15Tag         : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_15PLusTag      : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_PTB_U15Tag     : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_PTB_15PLusTag  : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_ETB_U15Tag     : np.zeros(year_len),
        TB_BDQ_LZD_NumDx_ETB_15PLusTag  : np.zeros(year_len),



        TB_AmikacynRifampicinResistant     : np.zeros((TB_PulLen, year_len)), 
        TB_EthionamideRifampicinResistant  : np.zeros((TB_PulLen, year_len)), 
        TB_PyrazinamideRifampicinResistant : np.zeros((TB_PulLen, year_len)), 

        TB_Rif_INH_Sen_SevereTag     : np.zeros((TB_ChildAdultLen, year_len)),
        TB_Rif_INH_Sen_NotSevereTag  : np.zeros((TB_ChildAdultLen, year_len)),
        TB_Rif_INH_Sen_MeningitisTag : np.zeros((TB_ChildAdultLen, year_len)),
        TB_Rif_INH_ResTag            : np.zeros((TB_ChildAdultLen, year_len)),
        TB_RR_FQ_SenTag              : np.zeros((TB_ChildAdultLen, year_len)),
        TB_Rif_FQ_Res_PlusTag        : np.zeros((TB_ChildAdultLen, year_len)),
        TB_BQD_LZD_Tag               : np.zeros((TB_ChildAdultLen, year_len)),
        TB_TxFL_Tag                  : np.zeros((TB_ChildAdultLen, year_len)),
        TB_TxSL_Tag                  : np.zeros((TB_ChildAdultLen, year_len)),
        TB_TxPreXDR_Tag              : np.zeros((TB_ChildAdultLen, year_len)),
        TB_LungSurgeryTag            : np.zeros((TB_ChildAdultLen, year_len)),
        

        TB_lPTB_ReferredDxTag           : np.zeros((TB_ChildAdultLen, year_len)),

        TB_HIVnClinicallyAssessedTag : np.zeros((TB_PulLen, TB_ChildAdultLen, year_len)),
        TB_HIVAnyReferredDxTag       : np.zeros((TB_PulLen, TB_ChildAdultLen, year_len)),

        # Notification
        TB_HisNotificationTag  : notif,
        TB_TargNotificationTag : notif,
        TB_ResNotificationTag  : np.array([notif, notif, notif*0]),

        # TB_ResPrevalenceTag    : ,# might come from pete dodds model,# might come from pete dodds model
        TB_ResHIVTBCoInfTag    : np.zeros((TB_HIVCoInfLen, year_len)),

        TB_HisIncidenceTag  : db['burden']['e_inc_num'][burden_first_idx:burden_final_idx+1],
        TB_HisMortalityTag  : db['burden']['e_mort_num'][burden_first_idx:burden_final_idx+1],
        TB_RetNrelPercTag   : db['Noti_Distribution']['ret_nrel_perc'],
        TB_ResIncidenceTag  : np.array([incidence, incidence, incidence*0]), 
        TB_ResMortalityTag  : np.array([mortality, mortality, mortality*0]), 
        TB_ResPrevalenceTag : np.array([prevalence, prevalence, prevalence*0]), 
        TB_UserEnteredIncidenceTag  : incidence,
        TB_UserEnteredMortalityTag  : mortality,
        TB_AlternativeProjBurdenTag : False,

        TB_FortIncidenceTag    : np.zeros((TB_ResultBounds, year_len)),
        TB_FortNotificationTag : np.zeros((TB_ResultBounds, year_len)),
        TB_FortPrevalenceTag   : np.zeros((TB_ResultBounds, year_len)),
        TB_FortMortalityTag    : np.zeros((TB_ResultBounds, year_len)),
        
        TB_SubnationalRatioTag  : 1.0, # 1.0 means it is a national model
        

        TB_ResPrevalencePctTag : db['projections']['prevalencePct'][first_idx:final_idx+1],

        TB_CounterfactualScenarioTag :  np.array([
                                    [0.76]*year_len,
                                    [0.00]*year_len,
                                    [0.84]*year_len,
                                ]
                                ),
        TB_CurrentScenarioTag        :  np.array([
                                    [0.76]*year_len,
                                    [0.30]*year_len,
                                    [0.84]*year_len,
                                ]
                                ),
                                
        TB_CounterfactualNotifTag: notif,
        
        TB_Baseline_HRi_allTag : np.full(year_len, 0.00),
        TB_Scaleup_HRi_allTag : np.full(year_len, 0.30),

        TB_NotificationByCaseTypeTag :  notif_split, # np.full((TB_NotificationCaseCount, year_len), 0),
        TB_PercNotificationTag       :  notif_pct,
        
        TB_FortVsnTag : '',
    
        TB_DynRatesTag : prm['r'],
        
        TB_DynPRMTag : prm,
        TB_DynUsedHIVIncdTag : used_HIV_incd,
        TB_DynProportionsTag: prm['p'],

        TB_DynUsedFitParamTag : {
            TB_DynRatesTag : r_include,
            TB_DynPRMTag : prm_include,
            TB_DynUsedHIVIncdTag : used_HIV_incd_include,
            TB_DynProportionsTag: p_include,
        },
        TB_DynRangesTag: {
            TB_DynRatesTag : r_range,
            TB_DynPRMTag : prm_range,
            TB_DynUsedHIVIncdTag : used_HIV_incd_range,
            TB_DynProportionsTag : p_range,
        },
                
        # TB_DynXSTag       : dyn_db['xs'],
        TB_DynPopDataTag  : dyn_db['data'],
        TB_DynamicalFirstYearTag : first_year,
        TB_DynamicalFinalYearTag : final_year,
        
        TB_FinalHistoricalYearTag: TB_FinalHist_Year, 
        TB_FortInputVersionTag: TB_FORT_INPUT_VERSION,
        TB_FortInputTag : fort_inputs,
        TB_FortInputScaleupTag : fort_inputs, # Default to same as inputs
        TB_FortOutputVersionTag : TB_FORT_OUTPUT_VERSION,
        TB_FortOutputTag: fort_outputs,

    }

# Helper function that will flatline the values from the database
# to the final year of the projection
def TB_flatlineDB(db, name, final_idx):
    if final_idx >= len(db['projections'][name]):#'notification'
        for i in range(len(db['projections'][name]), final_idx+1):
             db['projections'][name].append(db['projections'][name][-1])
             
def TB_flatlinBurden(db, name, final_idx):
    if final_idx >= len(db['burden'][name]):#'notification'
        for i in range(len(db['burden'][name]), final_idx+1):
             db['burden'][name].append(db['burden'][name][-1])


def TB_getPopDefaults(connection, countryID, subnatCode, base_year, final_year):    
    pop1CountryName = formatCountryFName(countryID, DPInitialConditionsDBVersion)#, subnatCode)
    #pop1CountryName = formatCountryFName(countryID, DPUPDVersion)#, subnatCode) 
    if GB_file_exists(connection, 'demproj', DPInitialConditionsDBSubDir + pop1CountryName):
        pop1 = np.array(GB_get_db_json(connection, 'demproj', DPInitialConditionsDBSubDir + pop1CountryName)['pop1']) 
        yr_slice = slice(base_year-GB_BaseYear, final_year-GB_BaseYear+1)

        pop = pop1[yr_slice] 
        pop_15_plus = pop[:, :, 15:, :, :].sum()
        return pop, pop_15_plus
    else:
        raise Exception('Country does not exist for TB Pop')
    

def calc_prevalence_from_notif_inci(notification, incidence):
    tx_mid = (2+0.2)/2
    ut_mid = (4+1)/2
    return  notification*tx_mid + (incidence - notification)*ut_mid
    